@@ -1,43 +1,22 @@
 from __future__ import annotations
 
+from collections.abc import Callable, Generator, Iterable, Iterator, Sequence, Set
 from datetime import datetime
-import enum
+from enum import Enum
 from functools import partial
 from itertools import chain
 from pathlib import Path
 from textwrap import dedent, fill
-from typing import (
-    TYPE_CHECKING,
-    AbstractSet,
-    Callable,
-    FrozenSet,
-    Generator,
-    Generic,
-    Iterable,
-    List,
-    Optional as O,
-    Sequence,
-    Tuple,
-    Type,
-    TypeVar,
-    Union,
-    overload,
-)
+from typing import overload
 
 import click
 
-from . import exceptions as E, models
-from .config import Config, setup_logging
+from . import manager as managers, models, results as E
+from .config import Config, Flavour, setup_logging
+from .plugins import load_plugins
 from .resolvers import Defn, MultiPkgModel, Strategy
 from .utils import TocReader, cached_property, get_version, is_outdated, tabulate, uniq
 
-if TYPE_CHECKING:
-    from .manager import CliManager
-
-    _R = TypeVar('_R')
-
-_EnumT = TypeVar('_EnumT', bound=enum.Enum)
-
 
 class Report:
     SUCCESS_SYMBOL = click.style('✓', fg='green')
@@ -46,7 +25,7 @@ class Report:
 
     def __init__(
         self,
-        results: Iterable[Tuple[Defn, E.ManagerResult]],
+        results: Iterable[tuple[Defn, E.ManagerResult]],
         filter_fn: Callable[[E.ManagerResult], bool] = lambda _: True,
     ):
         self.results = list(results)
@@ -70,14 +49,14 @@ class Report:
 
     def __str__(self) -> str:
         return '\n'.join(
-            f'{self._result_type_to_symbol(r)} {click.style(str(a), bold=True)}\n'
-            + fill(r.message, initial_indent=' ' * 2, subsequent_indent=' ' * 4)
+            f'{self._result_type_to_symbol(r)} {click.style(a.to_uri(), bold=True)}\n'
+            f'{fill(r.message, initial_indent=" " * 2, subsequent_indent=" " * 4)}'
             for a, r in self.results
             if self.filter_fn(r)
         )
 
-    def generate(self):
-        manager: CliManager = click.get_current_context().obj.m
+    def generate(self) -> None:
+        manager: managers.CliManager = click.get_current_context().obj.m
         if manager.config.auto_update_check:
             outdated, new_version = manager.run(is_outdated())
             if outdated:
@@ -87,15 +66,16 @@ class Report:
         if report:
             click.echo(report)
 
-    def generate_and_exit(self):
+    def generate_and_exit(self) -> None:
         self.generate()
         ctx = click.get_current_context()
         ctx.exit(self.exit_code)
 
 
-def _override_loop_policy() -> None:
-    # The proactor event loop which became the default on Windows in 3.8 is
-    # tripping up aiohttp<4.  See https://github.com/aio-libs/aiohttp/issues/4324
+def _override_asyncio_loop_policy() -> None:
+    # The proactor event loop which became the default loop on Windows
+    # in Python 3.8 is causing issues with aiohttp.
+    # See https://github.com/aio-libs/aiohttp/issues/4324
     import asyncio
 
     policy = getattr(asyncio, 'WindowsSelectorEventLoopPolicy', None)
@@ -103,34 +83,52 @@ def _override_loop_policy() -> None:
         asyncio.set_event_loop_policy(policy())
 
 
+def _set_asyncio_debug(debug: bool) -> None:
+    # This is the least obtrusive way to enable debugging in asyncio.
+    # Reference: https://docs.python.org/3/library/asyncio-dev.html#debug-mode
+    if debug:
+        import os
+
+        os.environ['PYTHONASYNCIODEBUG'] = '1'
+
+
 class ManagerWrapper:
     def __init__(self, ctx: click.Context):
         self.ctx = ctx
 
     @cached_property
-    def m(self) -> CliManager:
-        from .manager import CliManager
-
+    def m(self) -> managers.CliManager:
         try:
             config = Config.read(self.ctx.params['profile']).ensure_dirs()
         except FileNotFoundError:
-            config = self.ctx.invoke(configure)
+            config = self.ctx.invoke(configure, promptless=False)
+
         setup_logging(config, self.ctx.params['log_level'])
-        _override_loop_policy()
-        manager = CliManager.from_config(config)
+        _override_asyncio_loop_policy()
+        _set_asyncio_debug(self.ctx.params['log_level'] == 'DEBUG')
+
+        manager = managers.CliManager.from_config(config)
         return manager
 
 
-class PathParam(click.Path):
-    def coerce_path_result(self, value: str) -> Path:  # type: ignore
-        return Path(value)
+def _with_manager(
+    fn: Callable[..., object]
+) -> Callable[[click.Context, click.Parameter, object], object]:
+    def wrapper(
+        ctx: click.Context,
+        click_param: click.Parameter,
+        value: object,
+    ) -> object:
+        return fn(ctx.obj.m, value)
+
+    return wrapper
 
 
-class EnumParam(click.Choice, Generic[_EnumT]):
+class EnumParam(click.Choice):
     def __init__(
         self,
-        choice_enum: Type[_EnumT],
-        excludes: AbstractSet[_EnumT] = frozenset(),
+        choice_enum: type[Enum],
+        excludes: Set[Enum] = frozenset(),
         case_sensitive: bool = True,
     ) -> None:
         self.choice_enum = choice_enum
@@ -139,15 +137,22 @@ class EnumParam(click.Choice, Generic[_EnumT]):
             case_sensitive=case_sensitive,
         )
 
-    def convert(self, value: str, param: O[click.Parameter], ctx: O[click.Context]) -> _EnumT:
+    def convert(
+        self, value: str, param: click.Parameter | None, ctx: click.Context | None
+    ) -> Enum:
         parent_result = super().convert(value, param, ctx)
         return self.choice_enum[parent_result]
 
 
-def _callbackify(fn: Callable[..., _R]) -> Callable[[click.Context, click.Parameter, object], _R]:
-    return lambda c, _, v: fn(c.obj.m, v)
+def _register_plugin_commands(group: click.Group) -> click.Group:
+    plugin_hook = load_plugins()
+    additional_commands = (c for g in plugin_hook.instawow_add_commands() for c in g)
+    for command in additional_commands:
+        group.add_command(command)
+    return group
 
 
+@_register_plugin_commands
 @click.group(context_settings={'help_option_names': ('-h', '--help')})
 @click.version_option(get_version(), prog_name=__package__)
 @click.option(
@@ -165,26 +170,26 @@ def _callbackify(fn: Callable[..., _R]) -> Callable[[click.Context, click.Parame
     help='Activate the specified profile.',
 )
 @click.pass_context
-def main(ctx: click.Context, log_level: str, profile: str):
+def main(ctx: click.Context, log_level: str, profile: str) -> None:
     "Add-on manager for World of Warcraft."
     ctx.obj = ManagerWrapper(ctx)
 
 
 @overload
-def parse_into_defn(manager: CliManager, value: str, *, raise_invalid: bool = True) -> Defn:
+def parse_into_defn(manager: managers.Manager, value: str, *, raise_invalid: bool = True) -> Defn:
     ...
 
 
 @overload
 def parse_into_defn(
-    manager: CliManager, value: List[str], *, raise_invalid: bool = True
-) -> List[Defn]:
+    manager: managers.Manager, value: list[str], *, raise_invalid: bool = True
+) -> list[Defn]:
     ...
 
 
 def parse_into_defn(
-    manager: CliManager, value: Union[str, List[str]], *, raise_invalid: bool = True
-) -> Union[Defn, List[Defn]]:
+    manager: managers.Manager, value: str | list[str], *, raise_invalid: bool = True
+) -> Defn | list[Defn]:
     if not isinstance(value, str):
         defns = (parse_into_defn(manager, v, raise_invalid=raise_invalid) for v in value)
         return uniq(defns)
@@ -199,36 +204,36 @@ def parse_into_defn(
 
 
 def parse_into_defn_with_strategy(
-    manager: CliManager, value: Sequence[Tuple[Strategy, str]]
-) -> Iterable[Defn]:
+    manager: managers.Manager, value: Sequence[tuple[Strategy, str]]
+) -> Iterator[Defn]:
     defns = parse_into_defn(manager, [d for _, d in value])
     return map(Defn.with_strategy, defns, (s for s, _ in value))
 
 
 def parse_into_defn_with_version(
-    manager: CliManager, value: Sequence[Tuple[str, str]]
-) -> Iterable[Defn]:
+    manager: managers.Manager, value: Sequence[tuple[str, str]]
+) -> Iterator[Defn]:
     defns = parse_into_defn(manager, [d for _, d in value])
     return map(Defn.with_version, defns, (v for v, _ in value))
 
 
-def parse_into_defn_from_json_file(manager: CliManager, path: Path) -> Iterable[Defn]:
+def parse_into_defn_from_json_file(manager: managers.Manager, path: Path) -> Iterator[Defn]:
     faux_pkgs = MultiPkgModel.parse_file(path, encoding='utf-8')
     return map(Defn.from_pkg, faux_pkgs.__root__)
 
 
 def combine_addons(
-    fn: Callable[[CliManager, object], List[Defn]],
+    fn: Callable[[managers.Manager, object], Iterable[Defn]],
     ctx: click.Context,
     click_param: click.Parameter,
     value: object,
 ) -> None:
-    addons: List[Defn] = ctx.params.setdefault('addons', [])
+    addons: list[Defn] = ctx.params.setdefault('addons', [])
     if value:
         addons.extend(fn(ctx.obj.m, value))
 
 
-excluded_strategies = {Strategy.default, Strategy.version}
+_EXCLUDED_STRATEGIES = frozenset({Strategy.default, Strategy.version})
 
 
 @main.command()
@@ -238,7 +243,7 @@ excluded_strategies = {Strategy.default, Strategy.version}
 @click.option(
     '--import',
     '-i',
-    type=PathParam(dir_okay=False, exists=True),
+    type=click.Path(dir_okay=False, exists=True),
     expose_value=False,
     callback=partial(combine_addons, parse_into_defn_from_json_file),
     help='Install add-ons from the output of `list -f json`.',
@@ -247,13 +252,13 @@ excluded_strategies = {Strategy.default, Strategy.version}
     '--with-strategy',
     '-s',
     multiple=True,
-    type=(EnumParam(Strategy, excluded_strategies), str),
+    type=(EnumParam(Strategy, _EXCLUDED_STRATEGIES), str),
     expose_value=False,
     callback=partial(combine_addons, parse_into_defn_with_strategy),
     metavar='<STRATEGY ADDON>...',
     help='A strategy followed by an add-on definition.  '
     'The strategies are: '
-    f'{", ".join(s.name for s in Strategy if s not in excluded_strategies)}.',
+    f'{", ".join(s for s in Strategy if s not in _EXCLUDED_STRATEGIES)}.',
 )
 @click.option(
     '--version',
@@ -279,7 +284,7 @@ def install(obj: ManagerWrapper, addons: Sequence[Defn], replace: bool) -> None:
 
 
 @main.command()
-@click.argument('addons', nargs=-1, callback=_callbackify(parse_into_defn))
+@click.argument('addons', nargs=-1, callback=_with_manager(parse_into_defn))
 @click.pass_obj
 def update(obj: ManagerWrapper, addons: Sequence[Defn]) -> None:
     "Update installed add-ons."
@@ -302,16 +307,26 @@ def update(obj: ManagerWrapper, addons: Sequence[Defn]) -> None:
 
 
 @main.command()
-@click.argument('addons', nargs=-1, required=True, callback=_callbackify(parse_into_defn))
+@click.argument('addons', nargs=-1, required=True, callback=_with_manager(parse_into_defn))
+@click.option(
+    '--keep-folders',
+    is_flag=True,
+    default=False,
+    help="Do not delete the add-on folders.",
+)
 @click.pass_obj
-def remove(obj: ManagerWrapper, addons: Sequence[Defn]) -> None:
+def remove(obj: ManagerWrapper, addons: Sequence[Defn], keep_folders: bool) -> None:
     "Remove add-ons."
-    results = obj.m.run(obj.m.remove(addons))
+    results = obj.m.run(obj.m.remove(addons, keep_folders))
     Report(results.items()).generate_and_exit()
 
 
 @main.command()
-@click.argument('addon', callback=_callbackify(parse_into_defn))
+@click.argument('addon', callback=_with_manager(parse_into_defn))
+@click.option(
+    '--version',
+    help='Version to roll back to.',
+)
 @click.option(
     '--undo',
     is_flag=True,
@@ -319,17 +334,20 @@ def remove(obj: ManagerWrapper, addons: Sequence[Defn]) -> None:
     help='Undo rollback by reinstalling an add-on using the default strategy.',
 )
 @click.pass_context
-def rollback(ctx: click.Context, addon: Defn, undo: bool) -> None:
+def rollback(ctx: click.Context, addon: Defn, version: str | None, undo: bool) -> None:
     "Roll an add-on back to an older version."
     from .prompts import Choice, select
 
-    manager: CliManager = ctx.obj.m
-    limit = 10
+    manager: managers.CliManager = ctx.obj.m
+    version_limit = 10
+
+    if version and undo:
+        raise click.UsageError('Cannot use "--version" and "--undo" together')
 
     pkg = manager.get_pkg(addon)
     if not pkg:
         Report([(addon, E.PkgNotInstalled())]).generate_and_exit()
-        return  # noop
+        return  # pragma: no cover
 
     if not manager.resolvers[pkg.source].supports_rollback:
         Report(
@@ -339,30 +357,37 @@ def rollback(ctx: click.Context, addon: Defn, undo: bool) -> None:
     if undo:
         Report(manager.run(manager.update([addon], True)).items()).generate_and_exit()
 
-    versions = (
-        manager.database.query(models.PkgVersionLog)
-        .filter(
-            models.PkgVersionLog.pkg_source == pkg.source, models.PkgVersionLog.pkg_id == pkg.id
-        )
-        .order_by(models.PkgVersionLog.install_time.desc())
-        .limit(limit)
-        .all()
-    )
-    if len(versions) <= 1:
-        Report([(addon, E.PkgFileUnavailable('cannot find older versions'))]).generate_and_exit()
-
     reconstructed_defn = Defn.from_pkg(pkg)
-    choices = [
-        Choice(
-            [('', v.version)],
-            value=v.version,
-            disabled='installed version' if v.version == pkg.version else None,
+    if version:
+        selection = version
+    else:
+        versions = (
+            manager.database.query(models.PkgVersionLog)
+            .filter(
+                models.PkgVersionLog.pkg_source == pkg.source,
+                models.PkgVersionLog.pkg_id == pkg.id,
+            )
+            .order_by(models.PkgVersionLog.install_time.desc())
+            .limit(version_limit)
+            .all()
         )
-        for v in versions
-    ]
-    selection: str = select(
-        f'Select version of {reconstructed_defn} for rollback', choices
-    ).unsafe_ask()
+        if len(versions) <= 1:
+            Report(
+                [(addon, E.PkgFileUnavailable('cannot find older versions'))]
+            ).generate_and_exit()
+
+        choices = [
+            Choice(
+                [('', v.version)],
+                value=v.version,
+                disabled='installed version' if v.version == pkg.version else None,
+            )
+            for v in versions
+        ]
+        selection = select(
+            f'Select version of {reconstructed_defn.to_uri()} for rollback', choices
+        ).unsafe_ask()
+
     Report(
         manager.run(manager.update([reconstructed_defn.with_version(selection)], True)).items()
     ).generate_and_exit()
@@ -385,7 +410,6 @@ def reconcile(ctx: click.Context, auto: bool, list_unreconciled: bool) -> None:
         match_toc_ids,
         match_toc_names,
     )
-    from .models import is_pkg
     from .prompts import PkgChoice, confirm, select, skip
 
     preamble = dedent(
@@ -407,13 +431,13 @@ def reconcile(ctx: click.Context, auto: bool, list_unreconciled: bool) -> None:
         '''
     )
 
-    manager: CliManager = ctx.obj.m
+    manager: managers.CliManager = ctx.obj.m
 
-    def prompt_one(addons: List[AddonFolder], pkgs: List[models.Pkg]) -> Union[Defn, Tuple[()]]:
+    def prompt_one(addons: list[AddonFolder], pkgs: list[models.Pkg]) -> Defn | tuple[()]:
         def construct_choice(pkg: models.Pkg):
             defn = Defn.from_pkg(pkg)
             title = [
-                ('', f'=={defn}'),
+                ('', f'{defn.to_uri()}=='),
                 ('class:highlight-sub' if highlight_version else '', pkg.version),
             ]
             return PkgChoice(title, pkg=pkg, value=defn)
@@ -426,16 +450,16 @@ def reconcile(ctx: click.Context, auto: bool, list_unreconciled: bool) -> None:
         selection = select(f'{addon.name} [{addon.version or "?"}]', choices).unsafe_ask()
         return selection
 
-    def prompt(groups: Sequence[Tuple[List[AddonFolder], List[Defn]]]) -> Iterable[Defn]:
+    def prompt(groups: Sequence[tuple[list[AddonFolder], list[Defn]]]) -> Iterable[Defn]:
         uniq_defns = uniq(d for _, b in groups for d in b)
         results = manager.run(manager.resolve(uniq_defns))
         for addons, defns in groups:
-            shortlist: List[models.Pkg] = list(filter(is_pkg, (results[d] for d in defns)))
+            shortlist: list[models.Pkg] = list(filter(models.is_pkg, (results[d] for d in defns)))
             if shortlist:
                 selection = Defn.from_pkg(shortlist[0]) if auto else prompt_one(addons, shortlist)
                 selection and (yield selection)
 
-    def match_all() -> Generator[List[Defn], FrozenSet[AddonFolder], None]:
+    def match_all() -> Generator[list[Defn], frozenset[AddonFolder], None]:
         # Match in order of increasing heuristicitivenessitude
         for fn in (
             match_toc_ids,
@@ -492,11 +516,17 @@ def search(ctx: click.Context, search_terms: str, limit: int, sources: Sequence[
     "Search for add-ons to install."
     from .prompts import PkgChoice, checkbox, confirm
 
-    manager: CliManager = ctx.obj.m
+    manager: managers.CliManager = ctx.obj.m
 
-    pkgs = manager.run(manager.search(search_terms, limit, frozenset(sources) or None))
-    if pkgs:
-        choices = [PkgChoice(f'{p.name}  ({d}=={p.version})', d, pkg=p) for d, p in pkgs.items()]
+    entries = manager.run(manager.search(search_terms, limit, frozenset(sources) or None))
+    defns = [Defn(e.source, e.id) for e in entries]
+    pkgs = (
+        (d.with_(alias=r.slug), r)
+        for d, r in manager.run(manager.resolve(defns)).items()
+        if models.is_pkg(r)
+    )
+    choices = [PkgChoice(f'{p.name}  ({d.to_uri()}=={p.version})', d, pkg=p) for d, p in pkgs]
+    if choices:
         selections = checkbox('Select add-ons to install', choices=choices).unsafe_ask()
         if selections and confirm('Install selected add-ons?').unsafe_ask():
             ctx.invoke(install, addons=selections)
@@ -504,7 +534,7 @@ def search(ctx: click.Context, search_terms: str, limit: int, sources: Sequence[
         click.echo('No results found.')
 
 
-class ListFormats(enum.Enum):
+class ListFormats(str, Enum):
     simple = 'simple'
     detailed = 'detailed'
     json = 'json'
@@ -512,14 +542,14 @@ class ListFormats(enum.Enum):
 
 @main.command('list')
 @click.argument(
-    'addons', nargs=-1, callback=_callbackify(partial(parse_into_defn, raise_invalid=False))
+    'addons', nargs=-1, callback=_with_manager(partial(parse_into_defn, raise_invalid=False))
 )
 @click.option(
     '--format',
     '-f',
     'output_format',
     type=EnumParam(ListFormats),
-    default=ListFormats.simple.name,
+    default=ListFormats.simple,
     show_default=True,
     help='Change the output format.',
 )
@@ -532,16 +562,16 @@ def list_installed(
 
     def format_deps(pkg: models.Pkg):
         return (
-            str(d.with_(alias=p.slug) if p else d)
-            for d in pkg.deps
-            for d in (Defn(pkg.source, d.id),)
+            (d.with_(alias=p.slug) if p else d).to_uri()
+            for e in pkg.deps
+            for d in (Defn(pkg.source, e.id),)
             for d, p in ((d, obj.m.get_pkg(d)),)
         )
 
     def get_wowi_desc_from_toc(pkg: models.Pkg):
         if pkg.source == 'wowi':
             toc_reader = TocReader.from_parent_folder(obj.m.config.addon_dir / pkg.folders[0].name)
-            return toc_reader['Notes'].value
+            return toc_reader['Notes'] or ''
         else:
             return pkg.description
 
@@ -568,7 +598,7 @@ def list_installed(
     elif output_format is ListFormats.detailed:
         formatter = click.HelpFormatter(max_width=99)
         for pkg in pkgs:
-            with formatter.section(Defn.from_pkg(pkg)):
+            with formatter.section(Defn.from_pkg(pkg).to_uri()):
                 formatter.write_dl(
                     (
                         ('Name', pkg.name),
@@ -583,11 +613,11 @@ def list_installed(
                 )
         click.echo(formatter.getvalue(), nl=False)
     else:
-        click.echo(''.join(f'{Defn.from_pkg(p)}\n' for p in pkgs), nl=False)
+        click.echo(''.join(f'{Defn.from_pkg(p).to_uri()}\n' for p in pkgs), nl=False)
 
 
 @main.command(hidden=True)
-@click.argument('addon', callback=_callbackify(partial(parse_into_defn, raise_invalid=False)))
+@click.argument('addon', callback=_with_manager(partial(parse_into_defn, raise_invalid=False)))
 @click.pass_context
 def info(ctx: click.Context, addon: Defn) -> None:
     "Alias of `list -f detailed`."
@@ -595,7 +625,7 @@ def info(ctx: click.Context, addon: Defn) -> None:
 
 
 @main.command()
-@click.argument('addon', callback=_callbackify(partial(parse_into_defn, raise_invalid=False)))
+@click.argument('addon', callback=_with_manager(partial(parse_into_defn, raise_invalid=False)))
 @click.pass_obj
 def reveal(obj: ManagerWrapper, addon: Defn) -> None:
     "Bring an add-on up in your file manager."
@@ -606,7 +636,7 @@ def reveal(obj: ManagerWrapper, addon: Defn) -> None:
         Report([(addon, E.PkgNotInstalled())]).generate_and_exit()
 
 
-def _show_active_config(ctx: click.Context, _param: click.Parameter, value: bool):
+def _show_active_config(ctx: click.Context, _param: click.Parameter, value: bool) -> None:
     if value:
         click.echo(ctx.obj.m.config.json(indent=2))
         ctx.exit()
@@ -623,33 +653,38 @@ def _show_active_config(ctx: click.Context, _param: click.Parameter, value: bool
     callback=_show_active_config,
     help='Show the active configuration and exit.',
 )
+@click.option(
+    '--promptless',
+    is_flag=True,
+    default=False,
+    help='Do not prompt for input and derive the configuration from the environment.',
+)
 @click.pass_context
-def configure(ctx: click.Context) -> Config:
+def configure(ctx: click.Context, promptless: bool) -> Config:
     "Configure instawow."
-    from prompt_toolkit.completion import PathCompleter, WordCompleter
+    if promptless:
+        constructor = Config
+    else:
+        from .prompts import PydanticValidator, path, select
 
-    from .prompts import PydanticValidator, text
+        addon_dir = path(
+            'Add-on directory:',
+            only_directories=True,
+            validate=PydanticValidator(Config, 'addon_dir'),
+        ).unsafe_ask()
+        game_flavour = select(
+            'Game flavour:',
+            choices=sorted(Flavour.__members__, reverse=not Config.is_classic_folder(addon_dir)),
+        ).unsafe_ask()
+        constructor = partial(Config, addon_dir=addon_dir, game_flavour=game_flavour)
 
-    addon_dir = text(
-        'Add-on directory: ',
-        completer=PathCompleter(only_directories=True, expanduser=True),
-        validate=PydanticValidator(Config, 'addon_dir'),
-    ).unsafe_ask()
-    game_flavour = text(
-        'Game flavour: ',
-        default='classic' if Config.is_classic_folder(addon_dir) else 'retail',
-        completer=WordCompleter(['retail', 'classic']),
-        validate=PydanticValidator(Config, 'game_flavour'),
-    ).unsafe_ask()
-    config = Config(
-        profile=ctx.find_root().params['profile'], addon_dir=addon_dir, game_flavour=game_flavour
-    ).write()
+    config = constructor(profile=ctx.find_root().params['profile']).write()
     click.echo(f'Configuration written to: {config.config_file}')
     return config
 
 
 @main.group('weakauras-companion')
-def _weakauras_group():
+def _weakauras_group() -> None:
     "Manage your WeakAuras."
 
 
@@ -667,8 +702,6 @@ def build_weakauras_companion(obj: ManagerWrapper) -> None:
 @click.pass_obj
 def list_installed_wago_auras(obj: ManagerWrapper) -> None:
     "List WeakAuras installed from Wago."
-    from textwrap import fill
-
     from .wa_updater import BuilderConfig, WaCompanionBuilder
 
     config = BuilderConfig()
@@ -684,23 +717,26 @@ def list_installed_wago_auras(obj: ManagerWrapper) -> None:
 
 
 @main.command(hidden=True)
-@click.argument('filename', type=PathParam(dir_okay=False))
+@click.argument('filename', type=click.Path(dir_okay=False))
 @click.option(
     '--age-cutoff',
-    type=str,
-    default=None,
     callback=lambda _, __, v: v and datetime.fromisoformat(v),
 )
-def generate_catalogue(filename: Path, age_cutoff: O[datetime]) -> None:
+def generate_catalogue(filename: str, age_cutoff: datetime | None) -> None:
     "Generate the master catalogue."
     import asyncio
 
     from .resolvers import Catalogue
 
     catalogue = asyncio.run(Catalogue.collate(age_cutoff))
-    filename.write_text(catalogue.json(indent=2), encoding='utf-8')
-    filename.with_suffix(f'.compact{filename.suffix}').write_text(
-        catalogue.json(separators=(',', ':')), encoding='utf-8'
+    file = Path(filename)
+    file.write_text(
+        catalogue.json(indent=2),
+        encoding='utf-8',
+    )
+    file.with_suffix(f'.compact{file.suffix}').write_text(
+        catalogue.json(separators=(',', ':')),
+        encoding='utf-8',
     )
 
 
@@ -715,5 +751,6 @@ def listen(ctx: click.Context) -> None:
     dummy_config = Config.get_dummy_config(profile='__jsonrpc__').ensure_dirs()
     log_level = ctx.find_root().params['log_level']
     setup_logging(dummy_config, log_level)
-    _override_loop_policy()
+    _override_asyncio_loop_policy()
+    _set_asyncio_debug(log_level == 'DEBUG')
     asyncio.run(json_rpc_server.listen())
