@@ -2,42 +2,35 @@ from __future__ import annotations
 
 import asyncio
 from collections import defaultdict
+from collections.abc import (
+    AsyncIterator,
+    Awaitable,
+    Callable,
+    Iterable,
+    Iterator,
+    Mapping,
+    Sequence,
+    Set,
+)
 from contextlib import asynccontextmanager, contextmanager
 import contextvars as cv
-from functools import partial
 from itertools import chain, compress, filterfalse, repeat, starmap
 import json
 from pathlib import Path, PurePath
 from shutil import copy
 from tempfile import NamedTemporaryFile
-from typing import (
-    TYPE_CHECKING,
-    AbstractSet,
-    Any,
-    AsyncIterator,
-    Awaitable,
-    Callable,
-    DefaultDict,
-    Dict,
-    Iterable,
-    Iterator,
-    List,
-    NoReturn,
-    Optional as O,
-    Sequence,
-    Set,
-    Tuple,
-    Type,
-    TypeVar,
-    Union,
-)
+from typing import TYPE_CHECKING, Any, TypeVar
 
 from loguru import logger
+from typing_extensions import TypeAlias
 
-from . import DB_REVISION, exceptions as E
+from . import DB_REVISION, results as E
+from .config import Config
 from .models import Pkg, PkgFolder, PkgVersionLog, is_pkg
+from .plugins import load_plugins
 from .resolvers import (
     Catalogue,
+    CatalogueEntry,
     CurseResolver,
     Defn,
     GithubResolver,
@@ -67,21 +60,21 @@ from .utils import (
 
 if TYPE_CHECKING:
     import aiohttp
-    from prompt_toolkit.shortcuts import ProgressBar
-    from sqlalchemy.orm import Session as SqlaSession, sessionmaker
+    import prompt_toolkit.shortcuts
+    import sqlalchemy.orm
     from yarl import URL
 
-    from .config import Config
-
     _T = TypeVar('_T')
-    _ManagerT = TypeVar('_ManagerT', bound='Manager')
+    _FT = TypeVar('_FT', bound=Callable[..., Any])
+    _TManager = TypeVar('_TManager', bound='Manager')
+
+    _BaseResolverDict: TypeAlias = 'dict[str, Resolver]'
+else:
+    _BaseResolverDict = dict
 
 
 USER_AGENT = 'instawow (https://github.com/layday/instawow)'
 
-_web_client: cv.ContextVar[aiohttp.ClientSession] = cv.ContextVar('_web_client')
-_locks: cv.ContextVar[DefaultDict[str, asyncio.Lock]] = cv.ContextVar('_locks')
-
 
 AsyncNamedTemporaryFile = t(NamedTemporaryFile)
 copy_async = t(copy)
@@ -89,7 +82,7 @@ move_async = t(move)
 
 
 @asynccontextmanager
-async def _open_temp_writer() -> AsyncIterator[Tuple[Path, Callable[[bytes], Awaitable[int]]]]:
+async def _open_temp_writer() -> AsyncIterator[tuple[Path, Callable[[bytes], Awaitable[int]]]]:
     fh = await AsyncNamedTemporaryFile(delete=False)
     path = Path(fh.name)
     try:
@@ -103,7 +96,7 @@ async def _open_temp_writer() -> AsyncIterator[Tuple[Path, Callable[[bytes], Awa
 
 
 @contextmanager
-def _open_archive(path: PurePath) -> Iterator[Tuple[Set[str], Callable[[Path], None]]]:
+def _open_archive(path: PurePath) -> Iterator[tuple[set[str], Callable[[Path], None]]]:
     from zipfile import ZipFile
 
     ZIP_EXCLUDES = {
@@ -128,12 +121,13 @@ async def download_archive(manager: Manager, pkg: Pkg, *, chunk_size: int = 4096
         pkg.source, pkg.id, pkg.version, manager.config.game_flavour
     )
     if await t(dest.exists)():
-        logger.debug(f'{url} is cached at {dest}')
+        logger.debug(f'retrieving {url} from cache at {dest}')
     elif url.startswith('file://'):
         await copy_async(file_uri_to_path(url), dest)
     else:
-        kwargs = {'raise_for_status': True, 'trace_request_ctx': {'report_progress': True}}
-        async with manager.web_client.get(url, **kwargs) as response, _open_temp_writer() as (
+        async with manager.web_client.get(
+            url, raise_for_status=True, trace_request_ctx={'report_progress': True}
+        ) as response, _open_temp_writer() as (
             temp_path,
             write,
         ):
@@ -147,27 +141,29 @@ async def download_archive(manager: Manager, pkg: Pkg, *, chunk_size: int = 4096
 
 async def cache_response(
     manager: Manager,
-    url: Union[str, URL],
-    *timedelta_args: Any,
-    label: O[str] = None,
-    to_json: bool = True,
-    request_kwargs: Dict[str, Any] = {},
+    url: str | URL,
+    ttl: Mapping[str, float],
+    *,
+    label: str | None = None,
+    is_json: bool = True,
+    request_extra: Mapping[str, Any] = {},
 ) -> Any:
-    dest = manager.config.cache_dir / shasum(str(url), json.dumps(request_kwargs))
-    if await t(is_not_stale)(dest, *timedelta_args):
-        logger.debug(f'{url} is cached at {dest}')
-        text = await t(dest.read_text)(encoding='utf-8')
-    else:
-        method = request_kwargs.pop('method', 'GET')
-        kwargs: Dict[str, Any] = {'raise_for_status': True, **request_kwargs}
+    async def make_request():
+        kwargs = {'method': 'GET', 'url': url, 'raise_for_status': True, **request_extra}
         if label:
             kwargs = {**kwargs, 'trace_request_ctx': {'report_progress': True, 'label': label}}
-        async with manager.web_client.request(method, url, **kwargs) as response:
-            text = await response.text()
+        async with manager.web_client.request(**kwargs) as response:
+            return await response.text()
 
+    dest = manager.config.cache_dir / shasum(url, request_extra)
+    if await t(is_not_stale)(dest, ttl):
+        logger.debug(f'loading {url} from cache at {dest} (ttl: {ttl})')
+        text = await t(dest.read_text)(encoding='utf-8')
+    else:
+        text = await make_request()
         await t(dest.write_text)(text, encoding='utf-8')
 
-    return json.loads(text) if to_json else text
+    return json.loads(text) if is_json else text
 
 
 def _should_migrate(engine: Any) -> bool:
@@ -192,7 +188,7 @@ def _should_migrate(engine: Any) -> bool:
             return not current
 
 
-def prepare_database(config: Config) -> sessionmaker:
+def prepare_database(config: Config) -> sqlalchemy.orm.sessionmaker:
     from sqlalchemy import create_engine
     from sqlalchemy.orm import sessionmaker
 
@@ -241,37 +237,24 @@ def init_web_client(**kwargs: Any) -> aiohttp.ClientSession:
     return ClientSession(**kwargs)
 
 
-class _DummyLock:
-    async def __aenter__(self) -> None:
-        pass
-
-    async def __aexit__(self, *args: object) -> None:
-        pass
-
-
 @object.__new__
 class _DummyResolver(Resolver):
     strategies = set()
 
-    async def resolve(self, defns: Sequence[Defn]) -> Dict[Defn, E.PkgSourceInvalid]:
+    async def resolve(
+        self, defns: Sequence[Defn]
+    ) -> dict[Defn, Pkg | E.ManagerError | E.InternalError]:
         return dict.fromkeys(defns, E.PkgSourceInvalid())
 
 
-class _ResolverDict(Dict[str, Resolver]):
+class _ResolverDict(_BaseResolverDict):
     def __missing__(self, key: str) -> Resolver:
         return _DummyResolver
 
 
-def _error_out(error: BaseException) -> Callable[[], Awaitable[NoReturn]]:
-    async def inner() -> NoReturn:
-        raise error
-
-    return inner
-
-
 async def capture_manager_exc_async(
     awaitable: Awaitable[_T],
-) -> Union[_T, E.ManagerError, E.InternalError]:
+) -> _T | E.ManagerError | E.InternalError:
     "Capture and log an exception raised in a coroutine."
     from aiohttp import ClientError
 
@@ -287,11 +270,19 @@ async def capture_manager_exc_async(
         return E.InternalError(error)
 
 
+class _DummyLock:
+    async def __aenter__(self) -> None:
+        pass
+
+    async def __aexit__(self, *args: object) -> None:
+        pass
+
+
 def _with_lock(
     lock_name: str,
     manager_bound: bool = True,
-) -> Callable[[_T], _T]:
-    def outer(coro_fn: _T):
+) -> Callable[[_FT], _FT]:
+    def outer(coro_fn: _FT):
         async def inner(self: Manager, *args: object, **kwargs: object):
             key = f'{id(self)}_{lock_name}' if manager_bound else lock_name
             async with self.locks[key]:
@@ -302,82 +293,84 @@ def _with_lock(
     return outer
 
 
+_web_client: cv.ContextVar[aiohttp.ClientSession] = cv.ContextVar('_web_client')
+
+dummy_locks: defaultdict[str, Any] = defaultdict(_DummyLock)
+_locks: cv.ContextVar[defaultdict[str, asyncio.Lock]] = cv.ContextVar(
+    '_locks', default=dummy_locks
+)
+
+
 class Manager:
+    RESOLVERS = (
+        CurseResolver,
+        WowiResolver,
+        TukuiResolver,
+        GithubResolver,
+        InstawowResolver,
+    )
+
     def __init__(
         self,
         config: Config,
-        database: SqlaSession,
-        catalogue: O[Catalogue] = None,
-        resolver_classes: Sequence[Type[Resolver]] = (
-            CurseResolver,
-            WowiResolver,
-            TukuiResolver,
-            GithubResolver,
-            InstawowResolver,
-        ),
+        database: sqlalchemy.orm.Session,
+        *,
+        web_client: aiohttp.ClientSession | None = None,
+        locks: defaultdict[str, asyncio.Lock] | None = None,
     ) -> None:
         self.config = config
         self.database = database
+
+        plugin_hook = load_plugins()
+        resolver_classes = chain(
+            self.RESOLVERS, (r for g in plugin_hook.instawow_add_resolvers() for r in g)
+        )
         self.resolvers = _ResolverDict((r.source, r(self)) for r in resolver_classes)
-        self._catalogue = catalogue
+
+        if web_client is not None:
+            self.web_client = web_client
+        if locks is not None:
+            self.locks = locks
+
+        self._catalogue = None
 
     @classmethod
-    def from_config(cls: Type[_ManagerT], config: Config) -> _ManagerT:
+    def from_config(cls: type[_TManager], config: Config, **kwargs: Any) -> _TManager:
         session_factory = prepare_database(config)
-        return cls(config, session_factory())
+        return cls(config, session_factory(), **kwargs)
 
     @property
     def web_client(self) -> aiohttp.ClientSession:
         "The web client session."
-        try:
-            return _web_client.get()
-        except LookupError:
-            task = asyncio.current_task()
-            if task is None:
-                raise RuntimeError('no running task')
-
-            web_client = init_web_client()
-            task.add_done_callback(lambda _: asyncio.create_task(web_client.close()))
-            _web_client.set(web_client)
-            logger.debug(f'initialised default web client with id {id(web_client)}')
-            return web_client
+        return _web_client.get()
 
     @web_client.setter
     def web_client(self, value: aiohttp.ClientSession) -> None:
         _web_client.set(value)
 
     @property
-    def locks(self) -> DefaultDict[str, asyncio.Lock]:
-        "Keeping things syncin'."
-        try:
-            return _locks.get()
-        except LookupError:
-            locks: DefaultDict[str, Any] = defaultdict(_DummyLock)
-            _locks.set(locks)
-            logger.debug('using dummy lock factory')
-            return locks
+    def locks(self) -> defaultdict[str, asyncio.Lock]:
+        "Lock factory used to synchronise async operations."
+        return _locks.get()
 
     @locks.setter
-    def locks(self, value: DefaultDict[str, asyncio.Lock]) -> None:
+    def locks(self, value: defaultdict[str, asyncio.Lock]) -> None:
         _locks.set(value)
 
-    def pair_uri(self, value: str) -> O[Tuple[str, str]]:
-        "Attempt to extract the source from a URI."
+    def pair_uri(self, value: str) -> tuple[str, str] | None:
+        "Attempt to extract the package source and alias from a URI."
 
         def from_urn():
             source, alias = value.partition(':')[::2]
             if alias:
                 yield (source, alias)
 
-        url_pairs = filter(
+        url_pairs: Iterator[tuple[str, Any]] = filter(
             all, ((r.source, r.get_alias_from_url(value)) for r in self.resolvers.values())
         )
-        return next(
-            chain(url_pairs, from_urn()),  # type: ignore
-            None,
-        )
+        return next(chain(url_pairs, from_urn()), None)
 
-    def get_pkg(self, defn: Defn, partial_match: bool = False) -> O[Pkg]:
+    def get_pkg(self, defn: Defn, partial_match: bool = False) -> Pkg | None:
         "Retrieve an installed package from a definition."
         return (
             (
@@ -398,23 +391,10 @@ class Manager:
             or None
         )
 
-    def find_damaged_pkgs(self) -> List[Pkg]:
-        "Find packages with missing folders."
-        folders_in_db = {f.name for f in self.database.query(PkgFolder).all()}
-        folders_on_disk = {f.name for f in self.config.addon_dir.iterdir()}
-        folder_complement = folders_in_db - folders_on_disk
-        damaged_pkgs: List[Pkg] = (
-            self.database.query(Pkg)
-            .join(PkgFolder)
-            .filter(PkgFolder.name.in_(folder_complement))
-            .all()
-        )
-        return damaged_pkgs
-
     def install_pkg(self, pkg: Pkg, archive: Path, replace: bool) -> E.PkgInstalled:
         "Install a package."
         with _open_archive(archive) as (top_level_folders, extract):
-            installed_conflicts: List[Pkg] = (
+            installed_conflicts: list[Pkg] = (
                 self.database.query(Pkg)
                 .join(Pkg.folders)
                 .filter(PkgFolder.name.in_(top_level_folders))
@@ -426,7 +406,7 @@ class Manager:
             if replace:
                 trash(
                     [self.config.addon_dir / f for f in top_level_folders],
-                    dst=self.config.temp_dir,
+                    dest=self.config.temp_dir,
                     missing_ok=True,
                 )
             else:
@@ -450,7 +430,7 @@ class Manager:
     def update_pkg(self, old_pkg: Pkg, new_pkg: Pkg, archive: Path) -> E.PkgUpdated:
         "Update a package."
         with _open_archive(archive) as (top_level_folders, extract):
-            installed_conflicts: List[Pkg] = (
+            installed_conflicts: list[Pkg] = (
                 self.database.query(Pkg)
                 .join(Pkg.folders)
                 .filter(PkgFolder.pkg_source != new_pkg.source, PkgFolder.pkg_id != new_pkg.id)
@@ -468,7 +448,7 @@ class Manager:
 
             trash(
                 [self.config.addon_dir / f.name for f in old_pkg.folders],
-                dst=self.config.temp_dir,
+                dest=self.config.temp_dir,
                 missing_ok=True,
             )
             extract(self.config.addon_dir)
@@ -483,32 +463,33 @@ class Manager:
 
         return E.PkgUpdated(old_pkg, new_pkg)
 
-    def remove_pkg(self, pkg: Pkg) -> E.PkgRemoved:
+    def remove_pkg(self, pkg: Pkg, keep_folders: bool) -> E.PkgRemoved:
         "Remove a package."
-        trash(
-            [self.config.addon_dir / f.name for f in pkg.folders],
-            dst=self.config.temp_dir,
-            missing_ok=True,
-        )
+        if not keep_folders:
+            trash(
+                [self.config.addon_dir / f.name for f in pkg.folders],
+                dest=self.config.temp_dir,
+                missing_ok=True,
+            )
         self.database.delete(pkg)
         self.database.commit()
 
         return E.PkgRemoved(pkg)
 
-    @_with_lock('load master catalogue', False)
+    @_with_lock('load catalogue', False)
     async def synchronise(self) -> Catalogue:
-        "Fetch the master catalogue from the interwebs and load it."
+        "Fetch the catalogue from the interwebs and load it."
         if self._catalogue is None:
-            label = 'Synchronising master catalogue'
+            label = 'Synchronising catalogue'
             url = (
                 'https://raw.githubusercontent.com/layday/instawow-data/data/'
                 'master-catalogue-v2.compact.json'
             )  # v2
-            raw_catalogue = await cache_response(self, url, 4, 'hours', label=label)
+            raw_catalogue = await cache_response(self, url, {'hours': 4}, label=label)
             self._catalogue = Catalogue.parse_obj(raw_catalogue)
         return self._catalogue
 
-    async def _resolve_deps(self, results: Iterable[Any]) -> Dict[Defn, Any]:
+    async def _resolve_deps(self, results: Iterable[Any]) -> dict[Defn, Any]:
         """Resolve package dependencies.
 
         The resolver will not follow dependencies
@@ -530,13 +511,11 @@ class Manager:
         pretty_deps = {d.with_(alias=r.slug) if is_pkg(r) else d: r for d, r in deps.items()}
         return pretty_deps
 
-    async def resolve(self, defns: Sequence[Defn], with_deps: bool = False) -> Dict[Defn, Any]:
+    async def resolve(self, defns: Sequence[Defn], with_deps: bool = False) -> dict[Defn, Any]:
         "Resolve definitions into packages."
         if not defns:
             return {}
 
-        await self.synchronise()
-
         defns_by_source = bucketise(defns, key=lambda v: v.source)
         results = await gather(
             (self.resolvers[s].resolve(b) for s, b in defns_by_source.items()),
@@ -558,9 +537,8 @@ class Manager:
         self,
         search_terms: str,
         limit: int,
-        sources: O[AbstractSet[str]] = None,
-        strategy: Strategy = Strategy.default,
-    ) -> Dict[Defn, Pkg]:
+        sources: Set[str] | None = None,
+    ) -> list[CatalogueEntry]:
         "Search the master catalogue for packages by name."
         import heapq
 
@@ -596,13 +574,12 @@ class Manager:
             ),
             key=lambda v: v[0],
         )
-        defns = [Defn(i.source, i.id, strategy=strategy) for _, i in weighted_entries]
-        resolve_results = await self.resolve(defns)
-        pkgs_by_defn = {d.with_(alias=r.slug): r for d, r in resolve_results.items() if is_pkg(r)}
-        return pkgs_by_defn
+        return [e for _, e in weighted_entries]
 
     @_with_lock('change state')
-    async def install(self, defns: Sequence[Defn], replace: bool) -> Dict[Defn, E.ManagerResult]:
+    async def install(
+        self, defns: Sequence[Defn], replace: bool
+    ) -> dict[Defn, E.PkgInstalled | E.ManagerError | E.InternalError]:
         "Install packages from a definition list."
         # We'll weed out installed dependencies from results after resolving.
         # Doing it this way isn't particularly efficient but avoids having to
@@ -618,90 +595,86 @@ class Manager:
             (d, r): download_archive(self, r) for d, r in resolve_results.items() if is_pkg(r)
         }
         archives = await gather(installables.values(), capture_manager_exc_async)
-        result_coros = chain_dict(
+        results = chain_dict(
             defns,
-            _error_out(E.PkgAlreadyInstalled()),
-            ((d, _error_out(r)) for d, r in resolve_results.items()),
-            (
-                (
-                    d,
-                    partial(t(self.install_pkg), p, a, replace)
-                    if isinstance(a, PurePath)
-                    else _error_out(a),
-                )
+            E.PkgAlreadyInstalled(),
+            resolve_results.items(),
+            ((d, a) for (d, _), a in zip(installables, archives) if not isinstance(a, PurePath)),
+            [
+                (d, await capture_manager_exc_async(t(self.install_pkg)(p, a, replace)))
                 for (d, p), a in zip(installables, archives)
-            ),
+                if isinstance(a, PurePath)
+            ],
         )
-        results: Dict[Defn, Any] = {
-            d: await capture_manager_exc_async(c()) for d, c in result_coros.items()
-        }
         return results
 
     @_with_lock('change state')
     async def update(
         self, defns: Sequence[Defn], retain_strategy: bool
-    ) -> Dict[Defn, E.ManagerResult]:
+    ) -> dict[Defn, E.PkgUpdated | E.ManagerError | E.InternalError]:
         """Update installed packages from a definition list.
 
         A ``retain_strategy`` value of false will instruct ``update``
         to extract the strategy from the installed package; otherwise
         the ``Defn`` strategy will be used.
         """
-        # Begin by attaching the source ID to each ``Defn``
-        # from the corresponding installed package.  Using the ID has
-        # the benefit of resolving installed-but-renamed packages -
-        # the slug is transient but the ID is not
-        defns_to_pkgs = {
-            (Defn.from_pkg(p) if not retain_strategy and p else d.with_(id=p and p.id)): p
-            for d, p in zip(defns, (self.get_pkg(d) for d in defns))
+        defns_to_pkgs = {d: p for d in defns for d, p in ((d, self.get_pkg(d)),) if p}
+        resolve_defns = {
+            # Attach the source ID to each ``Defn`` from the
+            # corresponding installed package.  Using the ID has the benefit
+            # of resolving installed-but-renamed packages - the slug is
+            # transient but the ID isn't
+            d.with_(id=p.id) if retain_strategy else Defn.from_pkg(p): d
+            for d, p in defns_to_pkgs.items()
+        }
+        # Discard the reconstructed ``Defn``s
+        resolve_results = {
+            resolve_defns[d]: r for d, r in (await self.resolve(list(resolve_defns))).items()
         }
-        resolve_results = await self.resolve([d for d, p in defns_to_pkgs.items() if p])
         installables = {d: r for d, r in resolve_results.items() if is_pkg(r)}
         updatables = {
             (d, o, n): download_archive(self, n)
-            for (d, n), o in zip(installables.items(), (defns_to_pkgs[d] for d in installables))
-            if n.version != o.version  # type: ignore
+            for d, n in installables.items()
+            for d, o, n in ((d, defns_to_pkgs[d], n),)
+            if n.version != o.version
         }
         archives = await gather(updatables.values(), capture_manager_exc_async)
-        result_coros = chain_dict(
-            defns_to_pkgs,
-            _error_out(E.PkgNotInstalled()),
-            ((d, _error_out(r)) for d, r in resolve_results.items()),
-            (
-                (d, _error_out(E.PkgUpToDate(is_pinned=d.strategy is Strategy.version)))
-                for d in installables
-            ),
-            (
-                (
-                    d,
-                    partial(t(self.update_pkg), *p, a)
-                    if isinstance(a, PurePath)
-                    else _error_out(a),
-                )
-                for (d, *p), a in zip(updatables, archives)
-            ),
-        )
-        results: Dict[Defn, Any] = {
-            d: await capture_manager_exc_async(c()) for d, c in result_coros.items()
-        }
-        return results
-
-    @_with_lock('change state')
-    async def remove(self, defns: Sequence[Defn]) -> Dict[Defn, E.ManagerResult]:
-        "Remove packages by their definition."
-        maybe_pkgs = (self.get_pkg(d) for d in defns)
-        result_coros = chain_dict(
+        results = chain_dict(
             defns,
-            _error_out(E.PkgNotInstalled()),
-            ((d, partial(t(self.remove_pkg), p)) for d, p in zip(defns, maybe_pkgs) if p),
+            E.PkgNotInstalled(),
+            resolve_results.items(),
+            (
+                (d, E.PkgUpToDate(is_pinned=p.options.strategy == Strategy.version))
+                for d, p in installables.items()
+            ),
+            ((d, a) for (d, *_), a in zip(updatables, archives) if not isinstance(a, PurePath)),
+            [
+                (d, await capture_manager_exc_async(t(self.update_pkg)(o, n, a)))
+                for (d, o, n), a in zip(updatables, archives)
+                if isinstance(a, PurePath)
+            ],
         )
-        results: Dict[Defn, Any] = {
-            d: await capture_manager_exc_async(c()) for d, c in result_coros.items()
-        }
         return results
 
     @_with_lock('change state')
-    async def pin(self, defns: Sequence[Defn]) -> Dict[Defn, E.ManagerResult]:
+    async def remove(
+        self, defns: Sequence[Defn], keep_folders: bool
+    ) -> dict[Defn, E.PkgRemoved | E.ManagerError | E.InternalError]:
+        "Remove packages by their definition."
+        results = chain_dict(
+            defns,
+            E.PkgNotInstalled(),
+            [
+                (d, await capture_manager_exc_async(t(self.remove_pkg)(p, keep_folders)))
+                for d in defns
+                for d, p in ((d, self.get_pkg(d)),)
+                if p
+            ],
+        )
+        return results
+
+    @_with_lock('change state')
+    async def pin(self, defns: Sequence[Defn]) -> dict[Defn, E.ManagerResult]:
         """Pin and unpin installed packages.
 
         instawow does not have true pinning.  This sets the strategy
@@ -712,13 +685,13 @@ class Manager:
         package.
         """
 
-        strategies = {Strategy.default, Strategy.version}
+        strategies = frozenset({Strategy.default, Strategy.version})
 
-        def pin(defn: Defn, pkg: O[Pkg]) -> E.ManagerResult:
+        def pin(defn: Defn, pkg: Pkg | None) -> E.ManagerResult:
             if not pkg:
                 return E.PkgNotInstalled()
             elif {defn.strategy} <= strategies <= self.resolvers[pkg.source].strategies:
-                pkg.options.strategy = defn.strategy.name
+                pkg.options.strategy = defn.strategy
                 self.database.commit()
                 return E.PkgInstalled(pkg)
             else:
@@ -738,14 +711,9 @@ def _extract_filename_from_hdr(response: aiohttp.ClientResponse) -> str:
 
 
 def _init_cli_web_client(
-    bar: ProgressBar, tickers: Set[asyncio.Task[None]]
+    bar: prompt_toolkit.shortcuts.ProgressBar, tickers: set[asyncio.Task[None]]
 ) -> aiohttp.ClientSession:
-    from aiohttp import ClientResponse, TraceConfig, hdrs
-
-    if TYPE_CHECKING:
-
-        class TraceRequestEndParams:
-            response: ClientResponse
+    from aiohttp import TraceConfig, TraceRequestEndParams, hdrs
 
     async def do_on_request_end(
         client_session: aiohttp.ClientSession,
@@ -758,21 +726,20 @@ def _init_cli_web_client(
             return
 
         async def ticker() -> None:
-            label = (
-                ctx.get('label') or f'Downloading {_extract_filename_from_hdr(params.response)}'
-            )
-            total = params.response.content_length
+            response = params.response
+            label = ctx.get('label') or f'Downloading {_extract_filename_from_hdr(response)}'
+
+            total = response.content_length
             # The encoded size is not exposed in the streaming API and
             # ``content_length`` has the size of the payload after gzipping -
             # we can't know what the actual size of a file is in transfer
-            if params.response.headers.get(hdrs.CONTENT_ENCODING) == 'gzip':
+            if response.headers.get(hdrs.CONTENT_ENCODING) == 'gzip':
                 total = None
 
             counter = bar(label=label, total=total)
             try:
-                content = params.response.content
-                while not content.is_eof():
-                    counter.items_completed = content.total_bytes
+                while not response.content.is_eof():
+                    counter.items_completed = response.content.total_bytes
                     bar.invalidate()
                     await asyncio.sleep(tick_interval)
             finally:
@@ -781,13 +748,13 @@ def _init_cli_web_client(
         tickers.add(asyncio.create_task(ticker()))
 
     trace_config = TraceConfig()
-    trace_config.on_request_end.append(do_on_request_end)  # type: ignore
+    trace_config.on_request_end.append(do_on_request_end)
     trace_config.freeze()
     return init_web_client(trace_configs=[trace_config])
 
 
 @asynccontextmanager
-async def _cancel_tickers(tickers: Set[asyncio.Task[None]]):
+async def _cancel_tickers(tickers: set[asyncio.Task[None]]):
     try:
         yield
     finally:
