@@ -1,40 +1,27 @@
 from __future__ import annotations
 
+from collections.abc import AsyncIterator, Sequence, Set
 from datetime import datetime, timezone
-import enum
+from enum import Enum
 from itertools import chain, count, takewhile
 import re
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    AsyncIterable,
-    Callable,
-    ClassVar,
-    Dict,
-    FrozenSet,
-    List,
-    Optional as O,
-    Sequence,
-    Set,
-    Union,
-    cast,
-)
+import typing
+from typing import TYPE_CHECKING, Any, ClassVar, cast
 
 from pydantic import BaseModel
 from pydantic.datetime_parse import parse_datetime
 from typing_extensions import Literal, TypedDict
 from yarl import URL
 
-from . import exceptions as E, models as m
+from . import manager, models, results as E
+from .config import Flavour
 from .utils import bucketise, cached_property, gather, run_in_thread as t, uniq
 
 if TYPE_CHECKING:
     import aiohttp
 
-    from .manager import Manager as ManagerT
 
-
-class Strategy(enum.Enum):
+class Strategy(str, Enum):
     default = 'default'
     latest = 'latest'
     curse_latest_beta = 'curse_latest_beta'
@@ -57,16 +44,16 @@ class _HashableModel(BaseModel):
 class Defn(_HashableModel):
     source: str
     alias: str
-    id: O[str] = None
+    id: typing.Optional[str] = None
     strategy: Strategy = Strategy.default
-    version: O[str] = None
+    version: typing.Optional[str] = None
 
     @classmethod
-    def from_pkg(cls, pkg: Union[m.Pkg, PkgModel]) -> Defn:
+    def from_pkg(cls, pkg: models.Pkg | PkgModel) -> Defn:
         return cls(
             source=pkg.source,
-            id=pkg.id,
             alias=pkg.slug,
+            id=pkg.id,
             strategy=pkg.options.strategy,
             version=pkg.version,
         )
@@ -83,7 +70,7 @@ class Defn(_HashableModel):
     def with_version(self, version: str) -> Defn:
         return self.with_(strategy=Strategy.version, version=version)
 
-    def __str__(self) -> str:
+    def to_uri(self) -> str:
         return f'{self.source}:{self.alias}'
 
 
@@ -106,19 +93,19 @@ class _CatalogueEntryDefaultFields(TypedDict):
     id: str
     slug: str
     name: str
-    game_compatibility: Set[Literal['retail', 'classic']]
+    game_compatibility: Set[str]
     folders: Sequence[Sequence[str]]
     download_count: int
     last_updated: Any
 
 
-class _CatalogueEntry(BaseModel):
+class CatalogueEntry(BaseModel):
     source: str
     id: str
     slug: str
     name: str
-    game_compatibility: Set[Literal['retail', 'classic']]
-    folders: List[FrozenSet[str]]
+    game_compatibility: typing.Set[Flavour]
+    folders: typing.List[typing.Set[str]]
     download_count: int
     last_updated: datetime
     normalised_name: str
@@ -126,22 +113,19 @@ class _CatalogueEntry(BaseModel):
 
 
 class Catalogue(BaseModel):
-    __root__: List[_CatalogueEntry]
+    __root__: typing.List[CatalogueEntry]
 
     class Config:
-        json_encoders: Dict[type, Callable[..., Any]] = {
+        json_encoders: dict[type, Any] = {
             set: sorted,
-            frozenset: sorted,
         }
-        keep_untouched: Sequence[Any] = (cached_property,)
+        keep_untouched: tuple[Any, ...] = (cached_property,)
 
     @classmethod
-    async def collate(cls, age_cutoff: O[datetime]) -> Catalogue:
-        from .manager import init_web_client
-
+    async def collate(cls, age_cutoff: datetime | None) -> Catalogue:
         resolvers = (CurseResolver, WowiResolver, TukuiResolver, InstawowResolver)
 
-        async with init_web_client() as web_client:
+        async with manager.init_web_client() as web_client:
             items = [a for r in resolvers async for a in r.collect_items(web_client)]
 
         normalise = normalise_names()
@@ -150,7 +134,7 @@ class Catalogue(BaseModel):
             for s, i in bucketise(items, key=lambda v: v['source']).items()
         }
         entries = (
-            _CatalogueEntry(
+            CatalogueEntry(
                 **i,
                 normalised_name=normalise(i['name']),
                 derived_download_score=i['download_count']
@@ -164,7 +148,7 @@ class Catalogue(BaseModel):
         return catalogue
 
     @cached_property
-    def curse_slugs(self) -> Dict[str, str]:
+    def curse_slugs(self) -> dict[str, str]:
         return {a.slug: a.id for a in self.__root__ if a.source == 'curse'}
 
 
@@ -201,14 +185,14 @@ class PkgModel(_OrmModel):
     download_url: str
     date_published: datetime
     version: str
-    folders: List[_PkgModel_PkgFolder]
+    folders: typing.List[_PkgModel_PkgFolder]
     options: _PkgModel_PkgOptions
-    deps: List[_PkgModel_PkgDep]
-    logged_versions: List[_PkgModel_PkgVersion]
+    deps: typing.List[_PkgModel_PkgDep]
+    logged_versions: typing.List[_PkgModel_PkgVersion]
 
 
 class MultiPkgModel(BaseModel):
-    __root__: List[PkgModel]
+    __root__: typing.List[PkgModel]
 
 
 class Resolver:
@@ -216,7 +200,7 @@ class Resolver:
     name: ClassVar[str]
     strategies: ClassVar[Set[Strategy]]
 
-    def __init__(self, manager: ManagerT) -> None:
+    def __init__(self, manager: manager.Manager) -> None:
         self.manager = manager
 
     @property
@@ -225,26 +209,26 @@ class Resolver:
         return Strategy.version in self.strategies
 
     @staticmethod
-    def get_alias_from_url(value: str) -> O[str]:
+    def get_alias_from_url(value: str) -> str | None:
         "Attempt to extract a definition name from a given URL."
 
-    async def resolve(self, defns: Sequence[Defn]) -> Dict[Defn, Any]:
+    async def resolve(
+        self, defns: Sequence[Defn]
+    ) -> dict[Defn, models.Pkg | E.ManagerError | E.InternalError]:
         "Resolve add-on definitions into packages."
-        from .manager import capture_manager_exc_async
-
         results = await gather(
-            (self.resolve_one(d, None) for d in defns), capture_manager_exc_async
+            (self.resolve_one(d, None) for d in defns), manager.capture_manager_exc_async
         )
         return dict(zip(defns, results))
 
-    async def resolve_one(self, defn: Defn, metadata: O[Any]) -> m.Pkg:
+    async def resolve_one(self, defn: Defn, metadata: Any) -> models.Pkg:
         "Resolve an individual definition into a package."
         raise NotImplementedError
 
     @classmethod
     async def collect_items(
         cls, web_client: aiohttp.ClientSession
-    ) -> AsyncIterable[_CatalogueEntryDefaultFields]:
+    ) -> AsyncIterator[_CatalogueEntryDefaultFields]:
         "Yield add-ons from source for cataloguing."
         return
         yield
@@ -287,10 +271,10 @@ if TYPE_CHECKING:
         downloadUrl: str
         fileDate: str  # Upload datetime in ISO, e.g. '2020-02-02T12:12:12Z'
         releaseType: int  # 1 = stable; 2 = beta; 3 = alpha
-        dependencies: List[CurseAddon_FileDependency]
-        modules: List[CurseAddon_FileModules]
-        exposeAsAlternative: O[bool]
-        gameVersion: List[str]  # e.g. '8.3.0'
+        dependencies: list[CurseAddon_FileDependency]
+        modules: list[CurseAddon_FileModules]
+        exposeAsAlternative: bool | None
+        gameVersion: list[str]  # e.g. '8.3.0'
         gameVersionFlavor: Literal['wow_classic', 'wow_retail']
 
     class CurseAddon(TypedDict):
@@ -299,7 +283,7 @@ if TYPE_CHECKING:
         websiteUrl: str  # e.g. 'https://www.curseforge.com/wow/addons/molinari'
         summary: str  # One-line description of the add-on
         downloadCount: int  # Total number of downloads
-        latestFiles: List[CurseAddon_File]
+        latestFiles: list[CurseAddon_File]
         slug: str  # URL slug; 'molinari' in 'https://www.curseforge.com/wow/addons/molinari'
         dateReleased: str  # ISO datetime of latest release
 
@@ -307,20 +291,22 @@ if TYPE_CHECKING:
 class CurseResolver(Resolver):
     source = 'curse'
     name = 'CurseForge'
-    strategies = {
-        Strategy.default,
-        Strategy.latest,
-        Strategy.curse_latest_beta,
-        Strategy.curse_latest_alpha,
-        Strategy.any_flavour,
-        Strategy.version,
-    }
+    strategies = frozenset(
+        {
+            Strategy.default,
+            Strategy.latest,
+            Strategy.curse_latest_beta,
+            Strategy.curse_latest_alpha,
+            Strategy.any_flavour,
+            Strategy.version,
+        }
+    )
 
     # Reference: https://twitchappapi.docs.apiary.io/
     addon_api_url = URL('https://addons-ecs.forgesvc.net/api/v2/addon')
 
     @staticmethod
-    def get_alias_from_url(value: str) -> O[str]:
+    def get_alias_from_url(value: str) -> str | None:
         url = URL(value)
         if url.host == 'www.wowace.com' and len(url.parts) > 2 and url.parts[1] == 'projects':
             return url.parts[2].lower()
@@ -331,36 +317,36 @@ class CurseResolver(Resolver):
         ):
             return url.parts[3].lower()
 
-    async def resolve(self, defns: Sequence[Defn]) -> Dict[Defn, Any]:
+    async def resolve(
+        self, defns: Sequence[Defn]
+    ) -> dict[Defn, models.Pkg | E.ManagerError | E.InternalError]:
         from aiohttp import ClientResponseError
 
-        from .manager import cache_response, capture_manager_exc_async
-
         catalogue = await self.manager.synchronise()
 
-        defns_to_ids = {d: (d.id or catalogue.curse_slugs.get(d.alias) or d.alias) for d in defns}
-        numeric_ids = {i for i in defns_to_ids.values() if i.isdigit()}
+        defns_to_ids = {d: d.id or catalogue.curse_slugs.get(d.alias) or d.alias for d in defns}
+        numeric_ids = uniq(i for i in defns_to_ids.values() if i.isdigit())
         try:
-            json_response = await cache_response(
+            json_response: list[CurseAddon] = await manager.cache_response(
                 self.manager,
                 self.addon_api_url,
-                300,
-                request_kwargs={'method': 'POST', 'json': list(numeric_ids)},
+                {'minutes': 5},
+                request_extra={'method': 'POST', 'json': numeric_ids},
             )
         except ClientResponseError as error:
             if error.status != 404:
                 raise
             json_response = []
 
-        api_results: Dict[str, CurseAddon] = {str(r['id']): r for r in json_response}
+        api_results = {str(r['id']): r for r in json_response}
         results = await gather(
             (self.resolve_one(d, api_results.get(i)) for d, i in defns_to_ids.items()),
-            capture_manager_exc_async,
+            manager.capture_manager_exc_async,
         )
         return dict(zip(defns, results))
 
-    async def resolve_one(self, defn: Defn, metadata: O[CurseAddon]) -> m.Pkg:
-        if not metadata:
+    async def resolve_one(self, defn: Defn, metadata: CurseAddon | None) -> models.Pkg:
+        if metadata is None:
             raise E.PkgNonexistent
 
         if defn.strategy not in self.strategies:
@@ -368,12 +354,10 @@ class CurseResolver(Resolver):
 
         if defn.strategy is Strategy.version:
 
-            from .manager import cache_response
-
-            files = await cache_response(
+            files = await manager.cache_response(
                 self.manager,
                 self.addon_api_url / str(metadata['id']) / 'files',
-                3600,
+                {'hours': 1},
                 label=f'Fetching metadata from {self.name}',
             )
 
@@ -444,10 +428,10 @@ class CurseResolver(Resolver):
         except ValueError:
             raise E.PkgFileUnavailable(
                 f'no files compatible with {self.manager.config.game_flavour} '
-                f'using {defn.strategy.name!r} strategy'
+                f'using {defn.strategy} strategy'
             )
 
-        return m.Pkg(
+        return models.Pkg(
             source=self.source,
             id=str(metadata['id']),
             slug=metadata['slug'],
@@ -457,19 +441,20 @@ class CurseResolver(Resolver):
             download_url=file['downloadUrl'],
             date_published=parse_datetime(file['fileDate']),
             version=file['displayName'],
-            options=m.PkgOptions(strategy=defn.strategy.name),
-            deps=[m.PkgDep(id=str(d['addonId'])) for d in file['dependencies'] if d['type'] == 3],
+            options=models.PkgOptions(strategy=defn.strategy),
+            deps=[
+                models.PkgDep(id=str(d['addonId'])) for d in file['dependencies'] if d['type'] == 3
+            ],
         )
 
     @classmethod
     async def collect_items(
         cls, web_client: aiohttp.ClientSession
-    ) -> AsyncIterable[_CatalogueEntryDefaultFields]:
+    ) -> AsyncIterator[_CatalogueEntryDefaultFields]:
         classic_version_prefix = '1.13'
-        flavours = ('retail', 'classic')
 
-        def excise_flavours(files: List[CurseAddon_File]):
-            for c in flavours:
+        def excise_flavours(files: list[CurseAddon_File]):
+            for c in Flavour.__members__:
                 if any(f['gameVersionFlavor'] == f'wow_{c}' for f in files):
                     yield c
                 elif any(
@@ -487,7 +472,7 @@ class CurseResolver(Resolver):
                     gameId='1', sort=sort_order, pageSize=step, index=index
                 )
             ) as response:
-                json_response: List[CurseAddon] = await response.json()
+                json_response: list[CurseAddon] = await response.json()
 
             if not json_response:
                 break
@@ -510,54 +495,56 @@ class CurseResolver(Resolver):
                 )
 
 
-if TYPE_CHECKING:
+class WowiCommonTerms(TypedDict):
+    UID: str  # Unique add-on ID
+    UICATID: str  # ID of category add-on is placed in
+    UIVersion: str  # Add-on version
+    UIDate: int  # Upload date expressed as unix epoch
+    UIName: str  # User-facing add-on name
+    UIAuthorName: str
 
-    class WowiCommonTerms(TypedDict):
-        UID: str  # Unique add-on ID
-        UICATID: str  # ID of category add-on is placed in
-        UIVersion: str  # Add-on version
-        UIDate: int  # Upload date expressed as unix epoch
-        UIName: str  # User-facing add-on name
-        UIAuthorName: str
 
-    class WowiCompatibilityEntry(TypedDict):
-        version: str  # Game version, e.g. '8.3.0'
-        name: str  # Xpac or patch name, e.g. "Visions of N'Zoth" for 8.3.0
+class WowiCompatibilityEntry(TypedDict):
+    version: str  # Game version, e.g. '8.3.0'
+    name: str  # Xpac or patch name, e.g. "Visions of N'Zoth" for 8.3.0
 
-    class WowiListApiItem(WowiCommonTerms):
-        UIFileInfoURL: str  # Add-on page on WoWI
-        UIDownloadTotal: str  # Total number of downloads
-        UIDownloadMonthly: str  # Number of downloads in the last month and not 'monthly'
-        UIFavoriteTotal: str
-        UICompatibility: O[List[WowiCompatibilityEntry]]  # ``null`` if would be empty
-        UIDir: List[str]  # Names of folders contained in archive
-        UIIMG_Thumbs: O[List[str]]  # Thumbnail URLs; ``null`` if would be empty
-        UIIMGs: O[List[str]]  # Full-size image URLs; ``null`` if would be empty
-        # There are only two add-ons on the entire list with siblings
-        # (they refer to each other). I don't know if this was meant to capture
-        # dependencies (probably not) but it's so underused as to be worthless.
-        # ``null`` if would be empty
-        UISiblings: O[List[str]]
-        UIDonationLink: O[str]  # Absent from the first item on the list (!)
 
-    class WowiDetailsApiItem(WowiCommonTerms):
-        UIMD5: O[str]  # Archive hash, ``null` when UI is pending
-        UIFileName: str  # The actual filename, e.g. 'foo.zip'
-        UIDownload: str  # Download URL
-        UIPending: Literal['0', '1']  # Set to '1' if the file is awaiting approval
-        UIDescription: str  # Long description with BB Code and all
-        UIChangeLog: str  # This can also contain BB Code
-        UIHitCount: str  # Same as UIDownloadTotal
-        UIHitCountMonthly: str  # Same as UIDownloadMonthly
+class WowiListApiItem(WowiCommonTerms):
+    UIFileInfoURL: str  # Add-on page on WoWI
+    UIDownloadTotal: str  # Total number of downloads
+    UIDownloadMonthly: str  # Number of downloads in the last month and not 'monthly'
+    UIFavoriteTotal: str
+    UICompatibility: list[WowiCompatibilityEntry] | None  # ``null`` if would be empty
+    UIDir: list[str]  # Names of folders contained in archive
+    UIIMG_Thumbs: list[str] | None  # Thumbnail URLs; ``null`` if would be empty
+    UIIMGs: list[str] | None  # Full-size image URLs; ``null`` if would be empty
+    # There are only two add-ons on the entire list with siblings
+    # (they refer to each other). I don't know if this was meant to capture
+    # dependencies (probably not) but it's so underused as to be worthless.
+    # ``null`` if would be empty
+    UISiblings: list[str] | None
+    UIDonationLink: str | None  # Absent from the first item on the list (!)
 
-    class WowiCombinedItem(WowiListApiItem, WowiDetailsApiItem):
-        pass
+
+class WowiDetailsApiItem(WowiCommonTerms):
+    UIMD5: str | None  # Archive hash, ``null` when UI is pending
+    UIFileName: str  # The actual filename, e.g. 'foo.zip'
+    UIDownload: str  # Download URL
+    UIPending: Literal['0', '1']  # Set to '1' if the file is awaiting approval
+    UIDescription: str  # Long description with BB Code and all
+    UIChangeLog: str  # This can also contain BB Code
+    UIHitCount: str  # Same as UIDownloadTotal
+    UIHitCountMonthly: str  # Same as UIDownloadMonthly
+
+
+class WowiCombinedItem(WowiListApiItem, WowiDetailsApiItem):
+    pass
 
 
 class WowiResolver(Resolver):
     source = 'wowi'
     name = 'WoWInterface'
-    strategies = {Strategy.default}
+    strategies = frozenset({Strategy.default})
 
     # Reference: https://api.mmoui.com/v3/globalconfig.json
     # There's also a v4 API corresponding to the as yet unreleased Minion v4,
@@ -573,10 +560,8 @@ class WowiResolver(Resolver):
     list_api_url = 'https://api.mmoui.com/v3/game/WOW/filelist.json'
     details_api_url = URL('https://api.mmoui.com/v3/game/WOW/filedetails/')
 
-    list_api_items: O[Dict[str, WowiListApiItem]] = None
-
     @staticmethod
-    def get_alias_from_url(value: str) -> O[str]:
+    def get_alias_from_url(value: str) -> str | None:
         url = URL(value)
         if (
             url.host in {'wowinterface.com', 'www.wowinterface.com'}
@@ -584,38 +569,37 @@ class WowiResolver(Resolver):
             and url.parts[1] == 'downloads'
         ):
             if url.name == 'landing.php':
-                source_id = url.query.get('fileid')
-                if source_id:
-                    return source_id
+                return url.query.get('fileid')
             elif url.name == 'fileinfo.php':
-                source_id = url.query.get('id')
-                if source_id:
-                    return source_id
+                return url.query.get('id')
             else:
                 match = re.match(r'^(?:download|info)(?P<id>\d+)', url.name)
-                if match:
-                    return match.group('id')
+                return match and match.group('id')
 
-    async def resolve(self, defns: Sequence[Defn]) -> Dict[Defn, Any]:
+    async def _synchronise(self) -> dict[str, WowiListApiItem]:
+        async with self.manager.locks['load WoWI catalogue']:
+            list_api_items = await manager.cache_response(
+                self.manager,
+                self.list_api_url,
+                {'hours': 1},
+                label=f'Synchronising {self.name} catalogue',
+            )
+            return {i['UID']: i for i in list_api_items}
+
+    async def resolve(
+        self, defns: Sequence[Defn]
+    ) -> dict[Defn, models.Pkg | E.ManagerError | E.InternalError]:
         from aiohttp import ClientResponseError
 
-        from .manager import cache_response, capture_manager_exc_async
-
-        async with self.manager.locks['load WoWI catalogue']:
-            if self.list_api_items is None:
-                list_api_items = await cache_response(
-                    self.manager,
-                    self.list_api_url,
-                    3600,
-                    label=f'Synchronising {self.name} catalogue',
-                )
-                self.list_api_items = {i['UID']: i for i in list_api_items}
+        list_api_items = await self._synchronise()
 
         defns_to_ids = {d: ''.join(takewhile(str.isdigit, d.alias)) for d in defns}
-        numeric_ids = {i for i in defns_to_ids.values() if i.isdigit()}
+        numeric_ids = set(filter(None, defns_to_ids.values()))
         try:
-            details_api_items = await cache_response(
-                self.manager, self.details_api_url / f'{",".join(numeric_ids)}.json', 300
+            details_api_items: list[WowiDetailsApiItem] = await manager.cache_response(
+                self.manager,
+                self.details_api_url / f'{",".join(numeric_ids)}.json',
+                {'minutes': 5},
             )
         except ClientResponseError as error:
             if error.status != 404:
@@ -623,23 +607,23 @@ class WowiResolver(Resolver):
             details_api_items = []
 
         combined_items = {
-            i['UID']: cast('WowiCombinedItem', {**self.list_api_items[i['UID']], **i})
+            i['UID']: WowiCombinedItem(**{**list_api_items[i['UID']], **i})
             for i in details_api_items
         }
         results = await gather(
             (self.resolve_one(d, combined_items.get(i)) for d, i in defns_to_ids.items()),
-            capture_manager_exc_async,
+            manager.capture_manager_exc_async,
         )
         return dict(zip(defns, results))
 
-    async def resolve_one(self, defn: Defn, metadata: O[WowiCombinedItem]) -> m.Pkg:
-        if not metadata:
+    async def resolve_one(self, defn: Defn, metadata: WowiCombinedItem | None) -> models.Pkg:
+        if metadata is None:
             raise E.PkgNonexistent
 
         if defn.strategy not in self.strategies:
             raise E.PkgStrategyUnsupported(defn.strategy)
 
-        return m.Pkg(
+        return models.Pkg(
             source=self.source,
             id=metadata['UID'],
             slug=_slugify(f'{metadata["UID"]} {metadata["UIName"]}'),
@@ -649,16 +633,17 @@ class WowiResolver(Resolver):
             download_url=metadata['UIDownload'],
             date_published=parse_datetime(metadata['UIDate']),
             version=metadata['UIVersion'],
-            options=m.PkgOptions(strategy=defn.strategy.name),
+            options=models.PkgOptions(strategy=defn.strategy),
         )
 
     @classmethod
     async def collect_items(
         cls, web_client: aiohttp.ClientSession
-    ) -> AsyncIterable[_CatalogueEntryDefaultFields]:
+    ) -> AsyncIterator[_CatalogueEntryDefaultFields]:
         async with web_client.get(cls.list_api_url) as response:
-            list_api_items: List[WowiListApiItem] = await response.json()
+            list_api_items: list[WowiListApiItem] = await response.json()
 
+        flavours = set(Flavour.__members__)
         for list_item in list_api_items:
             yield _CatalogueEntryDefaultFields(
                 source=cls.source,
@@ -666,7 +651,7 @@ class WowiResolver(Resolver):
                 name=list_item['UIName'],
                 slug='',
                 folders=[list_item['UIDir']],
-                game_compatibility={'classic', 'retail'},
+                game_compatibility=flavours,
                 download_count=int(list_item['UIDownloadTotal']),
                 last_updated=list_item['UIDate'],
             )
@@ -704,7 +689,7 @@ if TYPE_CHECKING:
         # ISO *datetime* with space sep and without an offset, e.g. '2020-02-02 12:12:20'
         lastupdate: str
         name: str
-        patch: str
+        patch: str | None
         screenshot_url: str
         small_desc: str
         url: str
@@ -715,79 +700,114 @@ if TYPE_CHECKING:
 class TukuiResolver(Resolver):
     source = 'tukui'
     name = 'Tukui'
-    strategies = {Strategy.default}
+    strategies = frozenset({Strategy.default})
 
     # There's also a ``/client-api.php`` endpoint which is apparently
     # used by the Tukui client itself to check for updates for the two retail
     # UIs only.  The response body appears to be identical to ``/api.php``
     api_url = URL('https://www.tukui.org/api.php')
 
-    retail_uis = {'-1': 'tukui', '-2': 'elvui', 'tukui': 'tukui', 'elvui': 'elvui'}
-
     @staticmethod
-    def get_alias_from_url(value: str) -> O[str]:
+    def get_alias_from_url(value: str) -> str | None:
         url = URL(value)
-        if url.host == 'www.tukui.org' and url.path in {
-            '/addons.php',
-            '/classic-addons.php',
-            '/download.php',
-        }:
-            alias = url.query.get('id') or url.query.get('ui')
-            if alias:
-                return alias
+        if url.host == 'www.tukui.org':
+            if url.path in {'/addons.php', '/classic-addons.php'}:
+                return url.query.get('id')
+            elif url.path == '/download.php':
+                return url.query.get('ui')
+
+    async def _synchronise(self) -> dict[str, TukuiAddon | TukuiUi]:
+        async def fetch_ui(ui_slug: str):
+            addon: TukuiUi = await manager.cache_response(
+                self.manager,
+                self.api_url.with_query({'ui': ui_slug}),
+                {'minutes': 5},
+            )
+            return [(str(addon['id']), addon), (ui_slug, addon)]
+
+        async def fetch_addons(flavour: Flavour):
+            addons: list[TukuiAddon] = await manager.cache_response(
+                self.manager,
+                self.api_url.with_query(
+                    {'classic-addons' if flavour is Flavour.classic else 'addons': 'all'}
+                ),
+                {'minutes': 30},
+                label=f'Synchronising {self.name} {flavour} catalogue',
+            )
+            return [(str(a['id']), a) for a in addons]
+
+        async with self.manager.locks['load Tukui catalogue']:
+            requests = await gather(
+                ((fetch_ui('tukui'), fetch_ui('elvui')) if self.manager.config.is_retail else ())
+                + (fetch_addons(self.manager.config.game_flavour),)
+            )
+            return {k: v for l in requests for k, v in l}
+
+    async def resolve(
+        self, defns: Sequence[Defn]
+    ) -> dict[Defn, models.Pkg | E.ManagerError | E.InternalError]:
+        addons = await self._synchronise()
+
+        ids = (d.alias[: p if p != -1 else None] for d in defns for p in (d.alias.find('-', 1),))
+        results = await gather(
+            (self.resolve_one(d, addons.get(i)) for d, i in zip(defns, ids)),
+            manager.capture_manager_exc_async,
+        )
+        return dict(zip(defns, results))
+
+    async def resolve_one(self, defn: Defn, metadata: TukuiAddon | TukuiUi | None) -> models.Pkg:
+        if metadata is None:
+            raise E.PkgNonexistent
 
-    async def resolve_one(self, defn: Defn, metadata: None) -> m.Pkg:
         if defn.strategy not in self.strategies:
             raise E.PkgStrategyUnsupported(defn.strategy)
 
-        alias = ui_id = self.retail_uis.get(defn.alias)
-        if not alias:
-            alias = ''.join(takewhile('-'.__ne__, defn.alias))
-
-        if self.manager.config.is_classic:
-            query = 'classic-addon'
-        elif ui_id:
-            query = 'ui'
+        if metadata['id'] == -1:
+            slug = 'tukui'
+        elif metadata['id'] == -2:
+            slug = 'elvui'
         else:
-            query = 'addon'
+            slug = _slugify(f'{metadata["id"]} {metadata["name"]}')
 
-        url = self.api_url.with_query({query: alias})
-        async with self.manager.web_client.get(url) as response:
-            # Does not 404 for non-existent add-ons but the response body is empty
-            if not response.content_length:
-                raise E.PkgNonexistent
-            addon: Union[TukuiUi, TukuiAddon] = await response.json(content_type=None)  # text/html
-
-        return m.Pkg(
+        return models.Pkg(
             source=self.source,
-            id=str(addon['id']),
-            slug=ui_id or _slugify(f'{addon["id"]} {addon["name"]}'),
-            name=addon['name'],
-            description=addon['small_desc'],
-            url=addon['web_url'],
-            download_url=addon['url'],
-            date_published=datetime.fromisoformat(addon['lastupdate']).replace(
+            id=str(metadata['id']),
+            slug=slug,
+            name=metadata['name'],
+            description=metadata['small_desc'],
+            url=metadata['web_url'],
+            download_url=metadata['url'],
+            date_published=datetime.fromisoformat(metadata['lastupdate']).replace(
                 tzinfo=timezone.utc
             ),
-            version=addon['version'],
-            options=m.PkgOptions(strategy=defn.strategy.name),
+            version=metadata['version'],
+            options=models.PkgOptions(strategy=defn.strategy),
         )
 
     @classmethod
     async def collect_items(
         cls, web_client: aiohttp.ClientSession
-    ) -> AsyncIterable[_CatalogueEntryDefaultFields]:
-        for query, param in [
-            ('ui', 'tukui'),
-            ('ui', 'elvui'),
-            ('addons', 'all'),
-            ('classic-addons', 'all'),
-        ]:
-            async with web_client.get(cls.api_url.with_query({query: param})) as response:
-                metadata = await response.json(content_type=None)  # text/html
+    ) -> AsyncIterator[_CatalogueEntryDefaultFields]:
+        async def fetch_ui(ui_slug: str) -> tuple[set[Flavour], list[TukuiUi]]:
+            async with web_client.get(cls.api_url.with_query({'ui': ui_slug})) as response:
+                return ({Flavour.retail}, [await response.json(content_type=None)])  # text/html
 
-            items: List[Union[TukuiUi, TukuiAddon]] = [metadata] if query == 'ui' else metadata
-            game_compatibility = 'classic' if query == 'classic-addons' else 'retail'
+        async def fetch_addons(flavour: Flavour) -> tuple[set[Flavour], list[TukuiAddon]]:
+            async with web_client.get(
+                cls.api_url.with_query(
+                    {'classic-addons' if flavour is Flavour.classic else 'addons': 'all'}
+                )
+            ) as response:
+                return ({flavour}, await response.json(content_type=None))  # text/html
+
+        for flavours, items in await gather(
+            [
+                fetch_ui('tukui'),
+                fetch_ui('elvui'),
+                fetch_addons(Flavour.retail),
+                fetch_addons(Flavour.classic),
+            ]
+        ):
             for item in items:
                 yield _CatalogueEntryDefaultFields(
                     source=cls.source,
@@ -795,13 +815,13 @@ class TukuiResolver(Resolver):
                     slug='',
                     name=item['name'],
                     folders=[],
-                    game_compatibility={game_compatibility},
+                    game_compatibility=flavours,
                     # Split Tukui and ElvUI downloads evenly between them.
                     # They both have the exact same number of downloads so
                     # I'm assuming they're being counted together.
-                    # Anyway, this should help with scoring other add-ons
-                    # on the Tukui catalogue higher
-                    download_count=int(item['downloads']) // (2 if query == 'ui' else 1),
+                    # This should help with scoring other add-ons on the
+                    # Tukui catalogue higher
+                    download_count=int(item['downloads']) // (2 if item['id'] in {-1, -2} else 1),
                     last_updated=datetime.fromisoformat(item['lastupdate']).replace(
                         tzinfo=timezone.utc
                     ),
@@ -828,26 +848,23 @@ if TYPE_CHECKING:
     class GithubRelease(TypedDict):
         tag_name: str  # Hopefully the version
         published_at: str  # ISO datetime
-        assets: List[GithubReleaseAsset]
+        assets: list[GithubReleaseAsset]
 
 
 class GithubResolver(Resolver):
     source = 'github'
     name = 'GitHub'
-    strategies = {
-        Strategy.default,
-        Strategy.version,
-    }
+    strategies = frozenset({Strategy.default, Strategy.version})
 
     repos_api_url = URL('https://api.github.com/repos')
 
     @staticmethod
-    def get_alias_from_url(value: str) -> O[str]:
+    def get_alias_from_url(value: str) -> str | None:
         url = URL(value)
         if url.host == 'github.com' and len(url.parts) > 2:
             return '/'.join(url.parts[1:3])
 
-    async def resolve_one(self, defn: Defn, metadata: None) -> m.Pkg:
+    async def resolve_one(self, defn: Defn, metadata: None) -> models.Pkg:
         """Resolve a hypothetical add-on hosted on GitHub.
 
         The GitHub resolver is inspired by strongbox's
@@ -863,14 +880,14 @@ class GithubResolver(Resolver):
         """
         from aiohttp import ClientResponseError
 
-        from .manager import cache_response
-
         if defn.strategy not in self.strategies:
             raise E.PkgStrategyUnsupported(defn.strategy)
 
         repo_url = self.repos_api_url / defn.alias
         try:
-            project_metadata: GithubRepo = await cache_response(self.manager, repo_url, 3600)
+            project_metadata: GithubRepo = await manager.cache_response(
+                self.manager, repo_url, {'hours': 1}
+            )
         except ClientResponseError as error:
             if error.status == 404:
                 raise E.PkgNonexistent
@@ -914,7 +931,7 @@ class GithubResolver(Resolver):
         except StopIteration:
             raise E.PkgFileUnavailable
 
-        return m.Pkg(
+        return models.Pkg(
             source=self.source,
             id=project_metadata['full_name'],
             slug=project_metadata['full_name'].lower(),
@@ -924,21 +941,21 @@ class GithubResolver(Resolver):
             download_url=matching_asset['browser_download_url'],
             date_published=parse_datetime(release_metadata['published_at']),
             version=release_metadata['tag_name'],
-            options=m.PkgOptions(strategy=defn.strategy.name),
+            options=models.PkgOptions(strategy=defn.strategy),
         )
 
 
 class InstawowResolver(Resolver):
     source = 'instawow'
     name = 'instawow'
-    strategies = {Strategy.default}
+    strategies = frozenset({Strategy.default})
 
     _addons = {
         ('0', 'weakauras-companion'),
         ('1', 'weakauras-companion-autoupdate'),
     }
 
-    async def resolve_one(self, defn: Defn, metadata: None) -> m.Pkg:
+    async def resolve_one(self, defn: Defn, metadata: None) -> models.Pkg:
         if defn.strategy not in self.strategies:
             raise E.PkgStrategyUnsupported(defn.strategy)
 
@@ -953,7 +970,7 @@ class InstawowResolver(Resolver):
         if source_id == '1':
             await builder.build()
 
-        return m.Pkg(
+        return models.Pkg(
             source=self.source,
             id=source_id,
             slug=slug,
@@ -963,13 +980,13 @@ class InstawowResolver(Resolver):
             download_url=builder.addon_file.as_uri(),
             date_published=datetime.now(timezone.utc),
             version=(await t(builder.checksum)())[:7],
-            options=m.PkgOptions(strategy=defn.strategy.name),
+            options=models.PkgOptions(strategy=defn.strategy),
         )
 
     @classmethod
     async def collect_items(
         cls, web_client: aiohttp.ClientSession
-    ) -> AsyncIterable[_CatalogueEntryDefaultFields]:
+    ) -> AsyncIterator[_CatalogueEntryDefaultFields]:
         yield _CatalogueEntryDefaultFields(
             source=cls.source,
             id='1',
@@ -978,7 +995,7 @@ class InstawowResolver(Resolver):
             folders=[
                 ['WeakAurasCompanion'],
             ],
-            game_compatibility={'retail', 'classic'},
+            game_compatibility=set(Flavour.__members__),
             download_count=1,
             last_updated=datetime.now(timezone.utc),
         )
