@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import asyncio
 from collections import defaultdict, deque
+from collections.abc import Awaitable, Callable, Coroutine, Iterable, Iterator, Mapping, Sequence
 from contextlib import contextmanager
 from datetime import datetime, timedelta
 from functools import partial, wraps
@@ -10,70 +11,32 @@ from pathlib import Path, PurePath
 import posixpath
 from shutil import move as _move
 from tempfile import mkdtemp
-from typing import (
-    TYPE_CHECKING,
-    AbstractSet,
-    Any,
-    Awaitable,
-    Callable,
-    DefaultDict,
-    Deque,
-    Dict,
-    Generic,
-    Hashable,
-    Iterable,
-    Iterator,
-    List,
-    NamedTuple,
-    Optional as O,
-    Sequence,
-    Set,
-    Tuple,
-    Type,
-    TypeVar,
-    Union,
-    cast,
-    overload,
-)
+from typing import TYPE_CHECKING, Any, Generic, TypeVar, cast, overload
 
 if TYPE_CHECKING:
     from prompt_toolkit.shortcuts import ProgressBar
 
-    _H = TypeVar('_H', bound=Hashable)
-    _AnySet = TypeVar('_AnySet', bound=AbstractSet)
-
-
+_T = TypeVar('_T')
 _V = TypeVar('_V')
 
 
-class _TocEntry(NamedTuple):
-    key: str
-    value: str
-
-    def __bool__(self) -> bool:
-        return bool(self.value)
-
-
 class TocReader:
     """Extracts key–value pairs from TOC files."""
 
-    default = ''
-
     def __init__(self, contents: str) -> None:
-        possible_entries = (
-            map(str.strip, e.lstrip('#').partition(':')[::2])
+        self.entries = {
+            k: v
             for e in contents.splitlines()
             if e.startswith('##')
-        )
-        self.entries = {k: v for k, v in possible_entries if k}
+            for k, v in (map(str.strip, e.lstrip('#').partition(':')[::2]),)
+            if k
+        }
 
-    def __getitem__(self, key: Union[str, Tuple[str, ...]]) -> _TocEntry:
+    def __getitem__(self, key: str | tuple[str, ...]) -> str | None:
         if isinstance(key, tuple):
-            try:
-                return next(filter(None, (self[k] for k in key)))
-            except StopIteration:
-                key = key[0]
-        return _TocEntry(key, self.entries.get(key, self.default))
+            return next(filter(None, map(self.entries.get, key)), None)
+        else:
+            return self.entries.get(key)
 
     @classmethod
     def from_path(cls, path: Path) -> TocReader:
@@ -89,14 +52,14 @@ class cached_property(Generic[_V]):
         self.f = f
 
     @overload
-    def __get__(self, o: None, t: O[type] = None) -> cached_property[_V]:
+    def __get__(self, o: None, t: type[_T] | None = ...) -> cached_property[_V]:
         ...
 
     @overload
-    def __get__(self, o: Any, t: O[type] = None) -> _V:
+    def __get__(self, o: _T, t: type[_T] | None = ...) -> _V:
         ...
 
-    def __get__(self, o: Any, t: O[type] = None) -> Union[cached_property[_V], _V]:
+    def __get__(self, o: _T | None, t: type[_T] | None = None) -> cached_property[_V] | _V:
         if o is None:
             return self
         else:
@@ -104,29 +67,29 @@ class cached_property(Generic[_V]):
             return v
 
 
-def bucketise(iterable: Iterable[_V], key: Callable[[_V], _H]) -> DefaultDict[_H, List[_V]]:
+def bucketise(iterable: Iterable[_V], key: Callable[[_V], _T]) -> defaultdict[_T, list[_V]]:
     "Place the elements of an iterable in a bucket according to ``key``."
-    bucket: DefaultDict[Hashable, List[object]] = defaultdict(list)
+    bucket: defaultdict[_T, list[_V]] = defaultdict(list)
     for value in iterable:
         bucket[key(value)].append(value)
     return bucket
 
 
 def chain_dict(
-    keys: Iterable[_H], default: Any, *overrides: Iterable[Tuple[_H, _V]]
-) -> Dict[_H, _V]:
-    "Construct a dictionary from a series of iterables with overlapping keys."
+    keys: Iterable[_T], default: _V, *overrides: Iterable[tuple[_T, _V]]
+) -> dict[_T, _V]:
+    "Construct a dictionary from a series of two-tuple iterables with overlapping keys."
     return dict(chain(zip(keys, repeat(default)), *overrides))
 
 
-def uniq(it: Iterable[_H]) -> List[_H]:
+def uniq(it: Iterable[_T]) -> list[_T]:
     "Deduplicate hashable items in an iterable maintaining insertion order."
     return list(dict.fromkeys(it))
 
 
-def merge_intersecting_sets(it: Iterable[_AnySet]) -> Iterable[_AnySet]:
+def merge_intersecting_sets(it: Iterable[frozenset[_T]]) -> Iterator[frozenset[_T]]:
     "Recursively merge intersecting sets in a collection."
-    many_sets: Deque[AbstractSet[object]] = deque(it)
+    many_sets = deque(it)
     while True:
         try:
             this_set = many_sets.popleft()
@@ -145,20 +108,20 @@ def merge_intersecting_sets(it: Iterable[_AnySet]) -> Iterable[_AnySet]:
 
 
 @overload
-async def gather(
-    it: Iterable[Awaitable[_V]],
-    wrapper: Callable[..., Awaitable[_V]] = ...,
-) -> List[_V]:
+async def gather(it: Iterable[Awaitable[_V]], wrapper: None = ...) -> list[_V]:
     ...
 
 
 @overload
-async def gather(it: Iterable[Awaitable[_V]], wrapper: None = ...) -> List[_V]:
+async def gather(
+    it: Iterable[Awaitable[_V]],
+    wrapper: Callable[[Awaitable[_V]], Awaitable[_T]] = ...,
+) -> list[_T]:
     ...
 
 
 async def gather(
-    it: Iterable[Awaitable[_V]], wrapper: O[Callable[..., Awaitable[_V]]] = None
+    it: Iterable[Awaitable[_V]], wrapper: Callable[..., Awaitable[_V]] | None = None
 ) -> Sequence[_V]:
     if wrapper is not None:
         it = map(wrapper, it)
@@ -166,18 +129,18 @@ async def gather(
 
 
 @overload
-def run_in_thread(fn: Type[List[object]]) -> Callable[[Iterable[_V]], Awaitable[List[_V]]]:
+def run_in_thread(
+    fn: type[list[object]],
+) -> Callable[[Iterable[_V]], Coroutine[Any, Any, list[_V]]]:
     ...
 
 
 @overload
-def run_in_thread(fn: Callable[..., _V]) -> Callable[..., Awaitable[_V]]:
+def run_in_thread(fn: Callable[..., _V]) -> Callable[..., Coroutine[Any, Any, _V]]:
     ...
 
 
-def run_in_thread(
-    fn: Union[Type[List[object]], Callable[..., _V]]
-) -> Union[Callable[[Iterable[_V]], Awaitable[List[_V]]], Callable[..., Awaitable[_V]]]:
+def run_in_thread(fn: Callable[..., object]):
     @wraps(fn)
     def wrapper(*args: object, **kwargs: object):
         loop = asyncio.get_running_loop()
@@ -273,7 +236,7 @@ def make_progress_bar(**kwargs: Any) -> ProgressBar:
 
             if self._thread is not None:
 
-                def attempt_exit(sender: Application[object]):
+                def attempt_exit(sender: Application[None]):
                     sender.is_running and sender.exit()
 
                 self.app.on_invalidate = Event(self.app, attempt_exit)
@@ -313,32 +276,32 @@ def make_progress_bar(**kwargs: Any) -> ProgressBar:
     return progress_bar
 
 
-def move(src: PurePath, dst: PurePath) -> PurePath:
+def move(src: PurePath, dest: PurePath) -> PurePath:
     # See https://bugs.python.org/issue32689
-    return _move(str(src), dst)
+    return _move(str(src), dest)
 
 
-def trash(paths: Sequence[PurePath], *, dst: PurePath, missing_ok: bool = False) -> None:
-    parent_folder = Path(mkdtemp(dir=dst, prefix=f'deleted-{paths[0].name}-'))
+def trash(paths: Sequence[PurePath], *, dest: PurePath, missing_ok: bool = False) -> None:
+    parent_folder = Path(mkdtemp(dir=dest, prefix=f'deleted-{paths[0].name}-'))
     for path in paths:
         try:
-            move(path, dst=parent_folder)
+            move(path, dest=parent_folder)
         except (FileNotFoundError if missing_ok else ()):
             pass
 
 
-def shasum(*values: str) -> str:
+def shasum(*values: Any) -> str:
     "Base-16-encode a string using SHA-256 truncated to 32 characters."
     from hashlib import sha256
 
-    return sha256(''.join(values).encode()).hexdigest()[:32]
+    return sha256(''.join(map(str, values)).encode()).hexdigest()[:32]
 
 
-def is_not_stale(path: Path, ttl: int, unit: str = 'seconds') -> bool:
+def is_not_stale(path: Path, ttl: Mapping[str, float]) -> bool:
     "Check if a file is older than ``ttl``."
     mtime = path.exists() and path.stat().st_mtime
     return mtime > 0 and (
-        (datetime.now() - datetime.fromtimestamp(cast(float, mtime))) < timedelta(**{unit: ttl})
+        (datetime.now() - datetime.fromtimestamp(cast(float, mtime))) < timedelta(**ttl)
     )
 
 
@@ -352,7 +315,7 @@ def get_version() -> str:
         return __version__
 
 
-async def is_outdated() -> Tuple[bool, str]:
+async def is_outdated() -> tuple[bool, str]:
     """Check on PyPI to see if the installed copy of instawow is outdated.
 
     The response is cached for 24 hours.
@@ -362,7 +325,7 @@ async def is_outdated() -> Tuple[bool, str]:
     from .config import Config
     from .manager import init_web_client
 
-    def parse_version(version: str) -> Tuple[int, ...]:
+    def parse_version(version: str) -> tuple[int, ...]:
         version_numbers = version.split('.')[:3]
         int_only_version_numbers = (
             int(''.join(takewhile('0123456789'.__contains__, e))) for e in version_numbers
@@ -378,7 +341,7 @@ async def is_outdated() -> Tuple[bool, str]:
         return (False, '')
 
     cache_file = dummy_config.temp_dir / '.pypi_version'
-    if await run_in_thread(is_not_stale)(cache_file, 1, 'days'):
+    if await run_in_thread(is_not_stale)(cache_file, {'days': 1}):
         version = cache_file.read_text(encoding='utf-8')
     else:
         try:
@@ -394,12 +357,12 @@ async def is_outdated() -> Tuple[bool, str]:
     return (parse_version(version) > parse_version(__version__), version)
 
 
-def find_zip_base_dirs(names: Sequence[str]) -> Set[str]:
+def find_zip_base_dirs(names: Sequence[str]) -> set[str]:
     "Find top-level folders in a list of ZIP member paths."
     return {n for n in (posixpath.dirname(n) for n in names) if n and posixpath.sep not in n}
 
 
-def make_zip_member_filter(base_dirs: Set[str]) -> Callable[[str], bool]:
+def make_zip_member_filter(base_dirs: set[str]) -> Callable[[str], bool]:
     """Filter out items which are not sub-paths of top-level folders for the
     purpose of extracting ZIP files.
     """
