@@ -1,23 +1,11 @@
 from __future__ import annotations
 
+from collections.abc import Iterable, Iterator, Sequence
 from functools import partial, reduce
 from itertools import chain, product
 import time
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    ClassVar,
-    Dict,
-    Generic,
-    Iterable,
-    Iterator,
-    List,
-    Optional as O,
-    Sequence,
-    Tuple,
-    Type,
-    TypeVar,
-)
+import typing
+from typing import TYPE_CHECKING, Any, ClassVar, Generic, TypeVar
 
 from loguru import logger
 from pydantic import BaseModel, Field, validator
@@ -25,21 +13,26 @@ from pydantic.generics import GenericModel
 from typing_extensions import Literal, TypedDict
 from yarl import URL
 
+from . import manager
 from .config import BaseConfig
 from .utils import bucketise, chain_dict, gather, run_in_thread as t, shasum
 
 if TYPE_CHECKING:
-    from .manager import Manager
-
     ImportString = str
-    RemoteAuras = List[Tuple[List[WeakAura], WagoApiResponse, ImportString]]
+    RemoteAuras = Sequence[tuple[Sequence[WeakAura], WagoApiResponse, ImportString]]
+
+    _T = TypeVar('_T')
+
+    def type_(value: _T) -> type[_T]:
+        ...
+
+
+else:
+    type_ = type
 
 
 class BuilderConfig(BaseConfig):
-    wago_api_key: O[str]
-
-    class Config:  # type: ignore
-        env_prefix = 'INSTAWOW_'
+    wago_api_key: typing.Optional[str]
 
 
 WeakAuraT = TypeVar('WeakAuraT', bound='WeakAura')
@@ -51,7 +44,7 @@ class Auras(GenericModel, Generic[WeakAuraT]):
     _filename: ClassVar[str]
     _api_url: ClassVar[URL]
 
-    __root__: Dict[str, List[WeakAuraT]]
+    __root__: typing.Dict[str, typing.List[WeakAuraT]]
 
     class Config:
         arbitrary_types_allowed = True
@@ -64,18 +57,18 @@ class Auras(GenericModel, Generic[WeakAuraT]):
         raise NotImplementedError
 
     @classmethod
-    def merge(cls, *auras: Auras[WeakAuraT]) -> Iterable[Auras[WeakAuraT]]:
+    def merge(cls, *auras: Auras[Any]) -> Iterable[Auras[Any]]:
         "Merge auras of the same type."
         return (
             t(__root__=reduce(lambda a, b: {**a, **b}, (i.__root__ for i in a)))
-            for t, a in bucketise(auras, key=type).items()
+            for t, a in bucketise(auras, key=type_).items()
         )
 
 
 class WeakAura(BaseModel):
     id: str
     uid: str
-    parent: O[str]
+    parent: typing.Optional[str]
     url: URL
     version: int
 
@@ -83,7 +76,9 @@ class WeakAura(BaseModel):
         allow_population_by_field_name = True
         arbitrary_types_allowed = True
 
-    _convert_url = validator('url', pre=True)(lambda v: URL(v))
+    @validator('url', pre=True)
+    def _convert_url(cls, value: str) -> URL:
+        return URL(value)
 
 
 class WeakAuras(Auras[WeakAura]):
@@ -93,7 +88,8 @@ class WeakAuras(Auras[WeakAura]):
     @classmethod
     def from_lua_table(cls, lua_table: Any) -> WeakAuras:
         auras = (WeakAura.parse_obj(a) for a in lua_table['displays'].values() if a.get('url'))
-        return cls(__root__=bucketise(auras, key=lambda a: a.url.parts[1]))
+        sorted_auras = sorted(auras, key=lambda a: a.id)
+        return cls(__root__=bucketise(sorted_auras, key=lambda a: a.url.parts[1]))
 
 
 class Plateroo(WeakAura):
@@ -116,7 +112,8 @@ class Plateroos(Auras[Plateroo]):
             )
             if a.get('url')
         )
-        return cls(__root__={a.url.parts[1]: [a] for a in auras})
+        sorted_auras = sorted(auras, key=lambda a: a.id)
+        return cls(__root__={a.url.parts[1]: [a] for a in sorted_auras})
 
 
 if TYPE_CHECKING:
@@ -150,17 +147,17 @@ if TYPE_CHECKING:
 class WaCompanionBuilder:
     """A WeakAuras Companion port for shellfolk."""
 
-    def __init__(self, manager: Manager, builder_config: BuilderConfig) -> None:
+    def __init__(self, manager: manager.Manager, builder_config: BuilderConfig) -> None:
         self.manager = manager
         self.addon_file = self.manager.config.plugin_dir / __name__ / 'WeakAurasCompanion.zip'
         self.builder_config = builder_config
 
     @staticmethod
-    def extract_auras(model: Type[Auras[Any]], source: str) -> Auras[Any]:
-        from ._custom_slpp import slpp
+    def extract_auras(model: type[Auras[Any]], source: str) -> Auras[Any]:
+        from ._custom_slpp import decode
 
         source_after_assignment = source[source.find('=') + 1 :]
-        lua_table = slpp.decode(source_after_assignment)
+        lua_table = decode(source_after_assignment)
         return model.from_lua_table(lua_table)
 
     def extract_installed_auras(self) -> Iterator[Auras[Any]]:
@@ -180,7 +177,7 @@ class WaCompanionBuilder:
                 content = file.read_text(encoding='utf-8-sig', errors='replace')
                 aura_group_cache = self.manager.config.cache_dir / shasum(content)
                 if aura_group_cache.exists():
-                    logger.info(f'loading {file} from cache')
+                    logger.info(f'loading {file} from cache at {aura_group_cache}')
                     aura_groups = model.parse_file(aura_group_cache)
                 else:
                     start = time.perf_counter()
@@ -189,53 +186,54 @@ class WaCompanionBuilder:
                     aura_group_cache.write_text(aura_groups.json(), encoding='utf-8')
                 yield aura_groups
 
-    async def get_wago_metadata(self, aura_groups: Auras[Any]) -> List[WagoApiResponse]:
+    async def get_wago_metadata(self, aura_groups: Auras[Any]) -> list[WagoApiResponse]:
         from aiohttp import ClientResponseError
 
         from .manager import cache_response
 
         aura_ids = list(aura_groups.__root__)
         try:
-            return await cache_response(
-                self.manager,
-                aura_groups._api_url.with_query(ids=','.join(aura_ids)),
-                30,
-                'minutes',
-                label='Fetching aura metadata',
-                request_kwargs={'headers': {'api-key': self.builder_config.wago_api_key or ''}},
+            return sorted(
+                await cache_response(
+                    self.manager,
+                    aura_groups._api_url.with_query(ids=','.join(aura_ids)),
+                    {'minutes': 30},
+                    label='Fetching aura metadata',
+                    request_extra={'headers': {'api-key': self.builder_config.wago_api_key or ''}},
+                ),
+                key=lambda r: r['slug'],
             )
         except ClientResponseError as error:
             if error.status != 404:
                 raise
             return []
 
-    async def get_wago_import_string(self, aura_id: str) -> str:
+    async def get_wago_import_string(self, aura: WagoApiResponse) -> str:
         from .manager import cache_response
 
         return await cache_response(
             self.manager,
-            import_api_url.with_query(id=aura_id),
-            30,
-            'minutes',
-            label=f'Fetching aura with ID {aura_id}',
-            to_json=False,
-            request_kwargs={'headers': {'api-key': self.builder_config.wago_api_key or ''}},
+            import_api_url.with_query(id=aura['_id']),
+            {'minutes': 30},
+            label=f"Fetching aura '{aura['slug']}'",
+            is_json=False,
+            request_extra={'headers': {'api-key': self.builder_config.wago_api_key or ''}},
         )
 
     async def get_remote_auras(
         self, aura_groups: Auras[WeakAuraT]
-    ) -> Tuple[Type[Auras[WeakAuraT]], RemoteAuras]:
+    ) -> tuple[type[Auras[WeakAuraT]], RemoteAuras]:
         if not aura_groups.__root__:
             return (aura_groups.__class__, [])
 
         metadata = await self.get_wago_metadata(aura_groups)
-        import_strings = await gather(self.get_wago_import_string(r['_id']) for r in metadata)
+        import_strings = await gather(self.get_wago_import_string(r) for r in metadata)
         return (
             aura_groups.__class__,
-            list(filter(all, zip(aura_groups.__root__.values(), metadata, import_strings))),
+            [(aura_groups.__root__[r['slug']], r, i) for r, i in zip(metadata, import_strings)],
         )
 
-    def make_addon(self, auras: Sequence[Tuple[Type[Auras[Any]], RemoteAuras]]) -> None:
+    def make_addon(self, auras: Sequence[tuple[type[Auras[Any]], RemoteAuras]]) -> None:
         from importlib.resources import read_text
         from zipfile import ZipFile, ZipInfo
 
@@ -253,7 +251,7 @@ class WaCompanionBuilder:
         self.addon_file.parent.mkdir(exist_ok=True)
         with ZipFile(self.addon_file, 'w') as file:
 
-            def write_tpl(filename: str, ctx: Dict[str, Any]) -> None:
+            def write_tpl(filename: str, ctx: dict[str, Any]) -> None:
                 # Not using a plain string as the first argument to ``writestr``
                 # 'cause the timestamp would be set to the current time
                 # which would render the build unreproducible
@@ -321,7 +319,7 @@ class WaCompanionBuilder:
             write_tpl('init.lua', {})
             write_tpl(
                 'WeakAurasCompanion.toc',
-                {'interface': '11305' if self.manager.config.is_classic else '90001'},
+                {'interface': '11306' if self.manager.config.is_classic else '90002'},
             )
 
     async def build(self) -> None:
