@@ -1,15 +1,19 @@
 from functools import lru_cache
+from io import BytesIO
 import json
 import os
 from pathlib import Path
 import re
+from zipfile import ZipFile
 
 import pytest
 
-from instawow.config import Config
-from instawow.manager import Manager
+from instawow.config import Config, Flavour
+from instawow.manager import Manager, init_web_client
 from instawow.utils import get_version
 
+inf = float('inf')
+
 
 def pytest_addoption(parser):
     parser.addoption('--instawow-no-mock', action='store_true')
@@ -31,42 +35,52 @@ def should_mock(fn):
 
 
 @lru_cache(maxsize=None)
-def read_fixture(filename):
+def read_json_fixture(filename):
     return json.loads((Path(__file__).parent / 'fixtures' / filename).read_bytes())
 
 
 @lru_cache(maxsize=None)
-def make_zip(name):
-    from io import BytesIO
-    from zipfile import ZipFile
-
+def make_addon_zip(*folders):
     buffer = BytesIO()
     with ZipFile(buffer, 'w') as file:
-        file.writestr(f'{name}/{name}.toc', b'')
+        for folder in folders:
+            file.writestr(f'{folder}/{folder}.toc', b'')
+
     return buffer.getvalue()
 
 
 @pytest.fixture(scope='session', autouse=True)
-def temp_dir(tmp_path_factory):
+def iw_temp_dir(tmp_path_factory):
     temp_dir = os.environ['INSTAWOW_TEMP_DIR'] = str(tmp_path_factory.mktemp('temp'))
     yield temp_dir
 
 
-@pytest.fixture(params=['retail', 'classic'])
-def partial_config(tmp_path, request, temp_dir):
+@pytest.fixture(params=Flavour)
+def iw_partial_config(tmp_path, request, iw_temp_dir):
     addons = tmp_path / 'wow' / 'interface' / 'addons'
     addons.mkdir(parents=True)
-    return {'addon_dir': addons, 'temp_dir': temp_dir, 'game_flavour': request.param}
+    return {'addon_dir': addons, 'temp_dir': iw_temp_dir, 'game_flavour': request.param}
 
 
 @pytest.fixture
-def full_config(tmp_path, partial_config):
-    return {**partial_config, 'config_dir': tmp_path / 'config'}
+def iw_full_config(tmp_path, iw_partial_config):
+    return {**iw_partial_config, 'config_dir': tmp_path / 'config'}
 
 
 @pytest.fixture
-def manager(full_config):
-    yield Manager.from_config(Config(**full_config).write())
+def iw_config(iw_full_config):
+    yield Config(**iw_full_config).write()
+
+
+@pytest.fixture
+async def iw_web_client():
+    async with init_web_client() as web_client:
+        yield web_client
+
+
+@pytest.fixture
+def iw_manager(iw_config, iw_web_client):
+    yield Manager.from_config(iw_config, web_client=iw_web_client)
 
 
 @pytest.fixture
@@ -77,7 +91,7 @@ def mock_pypi(aresponses):
         '/pypi/instawow/json',
         'get',
         {'info': {'version': get_version()}},
-        repeat=float('inf'),
+        repeat=inf,
     )
 
 
@@ -88,8 +102,8 @@ def mock_master_catalogue(aresponses):
         'raw.githubusercontent.com',
         aresponses.ANY,
         'get',
-        read_fixture('master-catalogue.json'),
-        repeat=float('inf'),
+        read_json_fixture('master-catalogue.json'),
+        repeat=inf,
     )
 
 
@@ -100,22 +114,22 @@ def mock_curse(aresponses, mock_master_catalogue):
         'addons-ecs.forgesvc.net',
         '/api/v2/addon',
         'post',
-        read_fixture('curse-addon--all.json'),
-        repeat=float('inf'),
+        read_json_fixture('curse-addon--all.json'),
+        repeat=inf,
     )
     aresponses.add(
         'addons-ecs.forgesvc.net',
         '/api/v2/addon/20338/files',
         'get',
-        read_fixture('curse-addon-files.json'),
-        repeat=float('inf'),
+        read_json_fixture('curse-addon-files.json'),
+        repeat=inf,
     )
     aresponses.add(
         'edge.forgecdn.net',
         aresponses.ANY,
         'get',
-        aresponses.Response(body=make_zip('Molinari')),
-        repeat=float('inf'),
+        aresponses.Response(body=make_addon_zip('Molinari')),
+        repeat=inf,
     )
 
 
@@ -126,22 +140,22 @@ def mock_wowi(aresponses, mock_master_catalogue):
         'api.mmoui.com',
         '/v3/game/WOW/filelist.json',
         'get',
-        read_fixture('wowi-filelist.json'),
-        repeat=float('inf'),
+        read_json_fixture('wowi-filelist.json'),
+        repeat=inf,
     )
     aresponses.add(
         'api.mmoui.com',
         re.compile(r'^/v3/game/WOW/filedetails/'),
         'get',
-        read_fixture('wowi-filedetails.json'),
-        repeat=float('inf'),
+        read_json_fixture('wowi-filedetails.json'),
+        repeat=inf,
     )
     aresponses.add(
         'cdn.wowinterface.com',
         aresponses.ANY,
         'get',
-        aresponses.Response(body=make_zip('Molinari')),
-        repeat=float('inf'),
+        aresponses.Response(body=make_addon_zip('Molinari')),
+        repeat=inf,
     )
 
 
@@ -152,134 +166,143 @@ def mock_tukui(aresponses, mock_master_catalogue):
         'www.tukui.org',
         '/api.php?ui=tukui',
         'get',
-        read_fixture('tukui-ui--tukui.json'),
+        read_json_fixture('tukui-ui--tukui.json'),
         match_querystring=True,
-        repeat=float('inf'),
+        repeat=inf,
     )
     aresponses.add(
         'www.tukui.org',
-        '/api.php?addon=1',
+        '/api.php?ui=elvui',
         'get',
-        read_fixture('tukui-addon.json'),
+        read_json_fixture('tukui-ui--elvui.json'),
         match_querystring=True,
-        repeat=float('inf'),
+        repeat=inf,
     )
     aresponses.add(
         'www.tukui.org',
-        '/api.php?classic-addon=1',
+        '/api.php?addons=all',
         'get',
-        read_fixture('tukui-classic-addon.json'),
+        read_json_fixture('tukui-retail-addons.json'),
         match_querystring=True,
-        repeat=float('inf'),
+        repeat=inf,
+    )
+    aresponses.add(
+        'www.tukui.org',
+        '/api.php?classic-addons=all',
+        'get',
+        read_json_fixture('tukui-classic-addons.json'),
+        match_querystring=True,
+        repeat=inf,
     )
     aresponses.add(
         'www.tukui.org',
         '/api.php',
         'get',
         '',
-        repeat=float('inf'),
+        repeat=inf,
     )
     aresponses.add(
         'www.tukui.org',
         re.compile(r'^/downloads/tukui'),
         'get',
-        aresponses.Response(body=make_zip('Tukui')),
-        repeat=float('inf'),
+        aresponses.Response(body=make_addon_zip('Tukui')),
+        repeat=inf,
     )
     aresponses.add(
         'www.tukui.org',
         '/addons.php?download=1',
         'get',
-        aresponses.Response(body=make_zip('ElvUI_MerathilisUI')),
+        aresponses.Response(body=make_addon_zip('ElvUI_MerathilisUI')),
         match_querystring=True,
-        repeat=float('inf'),
+        repeat=inf,
     )
     aresponses.add(
         'www.tukui.org',
         '/classic-addons.php?download=1',
         'get',
-        aresponses.Response(body=make_zip('Tukui')),
+        aresponses.Response(body=make_addon_zip('Tukui')),
         match_querystring=True,
-        repeat=float('inf'),
+        repeat=inf,
     )
 
 
 @pytest.fixture
 @should_mock
-def mock_github(aresponses, mock_master_catalogue):
+def mock_github(aresponses):
     aresponses.add(
         'api.github.com',
         '/repos/AdiAddons/AdiButtonAuras',
         'get',
-        read_fixture('github-repo-lib-and-nolib.json'),
-        repeat=float('inf'),
+        read_json_fixture('github-repo-lib-and-nolib.json'),
+        repeat=inf,
     )
     aresponses.add(
         'api.github.com',
         '/repos/AdiAddons/AdiButtonAuras/releases/latest',
         'get',
-        read_fixture('github-release-lib-and-nolib.json'),
-        repeat=float('inf'),
+        read_json_fixture('github-release-lib-and-nolib.json'),
+        repeat=inf,
     )
     aresponses.add(
         'api.github.com',
         '/repos/AdiAddons/AdiButtonAuras/releases/tags/2.1.0',
         'get',
-        read_fixture('github-release-lib-and-nolib-older-version.json'),
-        repeat=float('inf'),
+        read_json_fixture('github-release-lib-and-nolib-older-version.json'),
+        repeat=inf,
     )
     aresponses.add(
         'api.github.com',
         '/repos/WeakAuras/WeakAuras2',
         'get',
-        read_fixture('github-repo-retail-and-classic.json'),
-        repeat=float('inf'),
+        read_json_fixture('github-repo-retail-and-classic.json'),
+        repeat=inf,
     )
     aresponses.add(
         'api.github.com',
         '/repos/WeakAuras/WeakAuras2/releases/latest',
         'get',
-        read_fixture('github-release-retail-and-classic.json'),
-        repeat=float('inf'),
+        read_json_fixture('github-release-retail-and-classic.json'),
+        repeat=inf,
     )
     aresponses.add(
         'api.github.com',
         '/repos/p3lim-wow/Molinari',
         'get',
-        read_fixture('github-repo-no-releases.json'),
-        repeat=float('inf'),
+        read_json_fixture('github-repo-no-releases.json'),
+        repeat=inf,
     )
     aresponses.add(
         'api.github.com',
         '/repos/p3lim-wow/Molinari/releases/latest',
         'get',
         aresponses.Response(body=b'', status=404),
-        repeat=float('inf'),
+        repeat=inf,
     )
     aresponses.add(
         'api.github.com',
         '/repos/AdiAddons/AdiButtonAuras/releases/tags/2.0.19',
         'get',
-        read_fixture('github-release-no-assets.json'),
-        repeat=float('inf'),
+        read_json_fixture('github-release-no-assets.json'),
+        repeat=inf,
     )
     aresponses.add(
         'api.github.com',
         '/repos/layday/foo-bar',
         'get',
         aresponses.Response(body=b'', status=404),
-        repeat=float('inf'),
+        repeat=inf,
     )
     aresponses.add(
         'github.com',
         re.compile(r'^(/[^/]*){2}/releases/download'),
         'get',
-        aresponses.Response(body=make_zip('Foo')),
-        repeat=float('inf'),
+        aresponses.Response(body=make_addon_zip('Foo')),
+        repeat=inf,
     )
 
 
 @pytest.fixture
+@should_mock
 def mock_all(
     mock_pypi,
     mock_master_catalogue,
